from datetime import time
from itertools import chain

from pandas.tseries.holiday import (
    USPresidentsDay,
    USLaborDay,
    USThanksgivingDay,
    GoodFriday
)
from pytz import timezone

from trading_calendars import TradingCalendar
from trading_calendars.trading_calendar import HolidayCalendar
from trading_calendars.us_holidays import (
    USMartinLutherKingJrAfter1998,
    USMemorialDay,
    USBlackFridayInOrAfter1993,
    USNewYearsDay,
    USIndependenceDay,
    Christmas,
    HurricaneSandyClosings,
    USNationalDaysofMourning,
)


class XCBFExchangeCalendar(TradingCalendar):
    """
    Exchange calendar for the CBOE Futures Exchange (XCBF).

    http://cfe.cboe.com/aboutcfe/expirationcalendar.aspx

    Open Time: 8:30am, America/Chicago
    Close Time: 3:15pm, America/Chicago

    (We are ignoring extended trading hours for now)
    """
    name = 'XCBF'

    tz = timezone("America/Chicago")

    open_times = (
        (None, time(8, 31)),
    )

    close_times = (
        (None, time(15, 15)),
    )

    @property
    def regular_holidays(self):
        return HolidayCalendar([
            USNewYearsDay,
            USMartinLutherKingJrAfter1998,
            USPresidentsDay,
            GoodFriday,
            USIndependenceDay,
            USMemorialDay,
            USLaborDay,
            USThanksgivingDay,
            Christmas
        ])

    @property
    def special_closes(self):
        return [(
            time(12, 15),
            HolidayCalendar([
                USBlackFridayInOrAfter1993,
            ])
        )]

    @property
    def adhoc_holidays(self):
        return list(chain(
            HurricaneSandyClosings,
            USNationalDaysofMourning,
        ))



"""
Tests for TradingCalendarDispatcher.
"""
from unittest import TestCase
from trading_calendars.errors import (
    CalendarNameCollision,
    CyclicCalendarAlias,
    InvalidCalendarName,
)
from trading_calendars.calendar_utils import TradingCalendarDispatcher
from trading_calendars.exchange_calendar_iepa import IEPAExchangeCalendar


class CalendarAliasTestCase(TestCase):

    @classmethod
    def setupClass(cls):
        # Make a calendar once so that we don't spend time in every test
        # instantiating calendars.
        cls.dispatcher_kwargs = dict(
            calendars={'IEPA': IEPAExchangeCalendar()},
            calendar_factories={},
            aliases={
                'IEPA_ALIAS': 'IEPA',
                'IEPA_ALIAS_ALIAS': 'IEPA_ALIAS',
            },
        )

    def setUp(self):
        self.dispatcher = TradingCalendarDispatcher(
            # Make copies here so that tests that mutate the dispatcher dicts
            # are isolated from one another.
            **{k: v.copy() for k, v in self.dispatcher_kwargs.items()}
        )

    def test_follow_alias_chain(self):
        self.assertIs(
            self.dispatcher.get_calendar('IEPA_ALIAS'),
            self.dispatcher.get_calendar('IEPA'),
        )
        self.assertIs(
            self.dispatcher.get_calendar('IEPA_ALIAS_ALIAS'),
            self.dispatcher.get_calendar('IEPA'),
        )

    def test_add_new_aliases(self):
        with self.assertRaises(InvalidCalendarName):
            self.dispatcher.get_calendar('NOT_IEPA')

        self.dispatcher.register_calendar_alias('NOT_IEPA', 'IEPA')

        self.assertIs(
            self.dispatcher.get_calendar('NOT_IEPA'),
            self.dispatcher.get_calendar('IEPA'),
        )

        self.dispatcher.register_calendar_alias(
            'IEPA_ALIAS_ALIAS_ALIAS',
            'IEPA_ALIAS_ALIAS'
        )
        self.assertIs(
            self.dispatcher.get_calendar('IEPA_ALIAS_ALIAS_ALIAS'),
            self.dispatcher.get_calendar('IEPA'),
        )

    def test_remove_aliases(self):
        self.dispatcher.deregister_calendar('IEPA_ALIAS_ALIAS')
        with self.assertRaises(InvalidCalendarName):
            self.dispatcher.get_calendar('IEPA_ALIAS_ALIAS')

    def test_reject_alias_that_already_exists(self):
        with self.assertRaises(CalendarNameCollision):
            self.dispatcher.register_calendar_alias('IEPA', 'NOT_IEPA')

        with self.assertRaises(CalendarNameCollision):
            self.dispatcher.register_calendar_alias('IEPA_ALIAS', 'NOT_IEPA')

    def test_allow_alias_override_with_force(self):
        self.dispatcher.register_calendar_alias('IEPA', 'NOT_IEPA', force=True)
        with self.assertRaises(InvalidCalendarName):
            self.dispatcher.get_calendar('IEPA')

    def test_reject_cyclic_aliases(self):
        add_alias = self.dispatcher.register_calendar_alias

        add_alias('A', 'B')
        add_alias('B', 'C')

        with self.assertRaises(CyclicCalendarAlias) as e:
            add_alias('C', 'A')

        expected = "Cycle in calendar aliases: ['C' -> 'A' -> 'B' -> 'C']"
        self.assertEqual(str(e.exception), expected)



#
# Copyright 2018 Quantopian, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import time
from pandas import Timestamp
from pandas.tseries.holiday import (
    DateOffset,
    EasterMonday,
    GoodFriday,
    Holiday,
    MO,
    previous_friday,
    weekend_to_monday,
)
from pytz import timezone

from .common_holidays import (
    new_years_day,
    christmas,
    weekend_christmas,
    boxing_day,
    weekend_boxing_day,
)
from .trading_calendar import (
    TradingCalendar,
    HolidayCalendar,
)

# Regular Holidays
# ----------------
# New Year's Day
LSENewYearsDay = new_years_day(observance=weekend_to_monday)

# Early May bank holiday
MayBank = Holiday(
    "Early May Bank Holiday",
    month=5,
    offset=DateOffset(weekday=MO(1)),
    day=1,
)
# Spring bank holiday is the last Monday in May except:
# - in 2002, it was moved to June 3
# - in 2012, it was moved to June 4
SpringBankBefore2002 = Holiday(
    "Spring Bank Holiday",
    month=5,
    day=31,
    offset=DateOffset(weekday=MO(-1)),
    end_date="2002-01-01",
)
SpringBank2002To2012 = Holiday(
    "Spring Bank Holiday",
    month=5,
    day=31,
    offset=DateOffset(weekday=MO(-1)),
    start_date="2003-01-01",
    end_date="2012-01-01",
)
SpringBank2013Onwards = Holiday(
    "Spring Bank Holiday",
    month=5,
    day=31,
    offset=DateOffset(weekday=MO(-1)),
    start_date="2013-01-01",
)
# Summer bank holiday
SummerBank = Holiday(
    "Summer Bank Holiday",
    month=8,
    day=31,
    offset=DateOffset(weekday=MO(-1)),
)

Christmas = christmas()

WeekendChristmas = weekend_christmas()

BoxingDay = boxing_day()

WeekendBoxingDay = weekend_boxing_day()

# Early Closes
# ------------
# If Christmas Eve falls on a weekday, that day is a half day.
# If it falls on a weekend, the preceding Friday is a half day.
ChristmasEve = Holiday(
    'Christmas Eve Early Close',
    month=12,
    day=24,
    observance=previous_friday,
)
# New Year's eve (or the preceding Friday if it falls on a weekend)
# is a half day.
NewYearsEve = Holiday(
    "New Year's Eve Early Close",
    month=12,
    day=31,
    observance=previous_friday,
)

# Ad Hoc Closes
# -------------
SpringBank2002 = Timestamp("2002-06-03", tz="UTC")
GoldenJubilee = Timestamp("2002-06-04", tz="UTC")
RoyalWedding = Timestamp("2011-04-29", tz="UTC")
SpringBank2012 = Timestamp("2012-06-04", tz="UTC")
DiamondJubilee = Timestamp("2012-06-05", tz="UTC")


class XLONExchangeCalendar(TradingCalendar):
    """
    Exchange calendar for the London Stock Exchange (XLON).

    Open Time: 8:00 AM, GMT
    Close Time: 4:30 PM, GMT

    Regularly-Observed Holidays:
    - New Years Day (observed on first business day on/after)
    - Good Friday
    - Easter Monday
    - Early May Bank Holiday (first Monday in May)
    - Spring Bank Holiday (last Monday in May)
    - Summer Bank Holiday (last Monday in May)
    - Christmas Day
    - Dec. 27th (if Christmas is on a weekend)
    - Boxing Day
    - Dec. 28th (if Boxing Day is on a weekend)

    Early Closes:
    - Christmas Eve
    - New Year's Eve
    """
    regular_early_close = time(12, 30)

    name = 'XLON'

    tz = timezone('Europe/London')

    open_times = (
        (None, time(8, 1)),
    )

    close_times = (
        (None, time(16, 30)),
    )

    @property
    def regular_holidays(self):
        return HolidayCalendar([
            LSENewYearsDay,
            GoodFriday,
            EasterMonday,
            MayBank,
            SpringBankBefore2002,
            SpringBank2002To2012,
            SpringBank2013Onwards,
            SummerBank,
            Christmas,
            WeekendChristmas,
            BoxingDay,
            WeekendBoxingDay
        ])

    @property
    def adhoc_holidays(self):
        return [
            SpringBank2002,
            GoldenJubilee,
            RoyalWedding,
            SpringBank2012,
            DiamondJubilee,
        ]

    @property
    def special_closes(self):
        return [
            (self.regular_early_close, HolidayCalendar([
                ChristmasEve,
                NewYearsEve,
            ]))
        ]



from unittest import TestCase
import pandas as pd

from .test_trading_calendar import EuronextCalendarTestBase
from trading_calendars.exchange_calendar_xlis import XLISExchangeCalendar


class XLISCalendarTestCase(EuronextCalendarTestBase, TestCase):

    answer_key_filename = 'xlis'
    calendar_class = XLISExchangeCalendar

    # The XLIS is open from 8:00 am to 4:30 pm.
    MAX_SESSION_HOURS = 8.5
    TIMEDELTA_TO_NORMAL_CLOSE = pd.Timedelta(hours=16, minutes=30)

    TZ = 'Europe/Lisbon'

    def test_old_holidays(self):
        """
        Test the before and after of holidays that are no longer observed.
        """
        all_sessions = self.calendar.all_sessions

        expected_holidays = [
            pd.Timestamp('2002-02-12', tz='UTC'),  # Carnival
            pd.Timestamp('2002-05-30', tz='UTC'),  # Corpus Christi Day
            pd.Timestamp('2002-04-25', tz='UTC'),  # Liberty Day
            pd.Timestamp('2002-06-10', tz='UTC'),  # Portugal Day
            pd.Timestamp('2001-06-13', tz='UTC'),  # Saint Anthony's Day
            pd.Timestamp('2002-08-15', tz='UTC'),  # Assumption Day
            pd.Timestamp('2001-10-05', tz='UTC'),  # Republic Day
            pd.Timestamp('2002-11-01', tz='UTC'),  # All Saints Day
            pd.Timestamp('2000-12-01', tz='UTC'),  # Independence Day
            pd.Timestamp('2000-12-08', tz='UTC'),  # Immaculate Conception
            pd.Timestamp('2002-12-24', tz='UTC'),  # Christmas Eve
        ]

        for session_label in expected_holidays:
            self.assertNotIn(session_label, all_sessions)

        expected_sessions = [
            pd.Timestamp('2003-03-04', tz='UTC'),  # Carnival
            pd.Timestamp('2003-06-16', tz='UTC'),  # Corpus Christi Day
            pd.Timestamp('2003-04-25', tz='UTC'),  # Liberty Day
            pd.Timestamp('2003-06-10', tz='UTC'),  # Portugal Day
            pd.Timestamp('2002-06-13', tz='UTC'),  # Saint Anthony's Day
            pd.Timestamp('2003-08-15', tz='UTC'),  # Assumption Day
            pd.Timestamp('2004-10-05', tz='UTC'),  # Republic Day
            pd.Timestamp('2004-11-01', tz='UTC'),  # All Saints Day
            pd.Timestamp('2003-12-01', tz='UTC'),  # Independence Day
            pd.Timestamp('2003-12-08', tz='UTC'),  # Immaculate Conception
            pd.Timestamp('2003-12-24', tz='UTC'),  # Christmas Eve
        ]

        for session_label in expected_sessions:
            self.assertIn(session_label, all_sessions)



from unittest import TestCase
import pandas as pd

from .test_trading_calendar import ExchangeCalendarTestBase
from trading_calendars.exchange_calendar_xasx import XASXExchangeCalendar


class XASXCalendarTestCase(ExchangeCalendarTestBase, TestCase):

    answer_key_filename = 'xasx'
    calendar_class = XASXExchangeCalendar

    # The XASX is open from 10:00 am to 4:00 pm.
    MAX_SESSION_HOURS = 6

    def test_normal_year(self):
        expected_holidays = [
            pd.Timestamp('2018-01-01', tz='UTC'),  # New Year's Day
            pd.Timestamp('2018-01-26', tz='UTC'),  # Australia Day
            pd.Timestamp('2018-03-30', tz='UTC'),  # Good Friday
            pd.Timestamp('2018-04-02', tz='UTC'),  # Easter Monday
            pd.Timestamp('2018-04-25', tz='UTC'),  # Anzac Day
            pd.Timestamp('2018-06-11', tz='UTC'),  # Queen's Birthday
            pd.Timestamp('2018-12-25', tz='UTC'),  # Christmas
            pd.Timestamp('2018-12-26', tz='UTC'),  # Boxing Day
        ]

        for session_label in expected_holidays:
            self.assertNotIn(session_label, self.calendar.all_sessions)

        early_closes = [
            pd.Timestamp('2018-12-24', tz='UTC'),  # Day before Christmas
            pd.Timestamp('2018-12-31', tz='UTC'),  # Day before New Year's
        ]

        for early_close_session_label in early_closes:
            self.assertIn(
                early_close_session_label,
                self.calendar.early_closes,
            )

    def test_holidays_fall_on_weekend(self):
        """
        Holidays falling on a weekend should be made up on the next weekday.

        Anzac Day is observed on the following Monday only when falling
        on a Sunday. In years where Anzac Day falls on a Saturday, there
        is no make-up.

        Christmas/Boxing Day are special cases, whereby if Christmas is a
        Saturday and Boxing Day is a Sunday, the next Monday and Tuesday will
        be holidays. If Christmas is a Sunday and Boxing Day is a Monday then
        Monday and Tuesday will still both be holidays.
        """
        expected_holidays = [
            # New Year's Day on a Sunday, observed on Monday.
            pd.Timestamp('2017-01-02', tz='UTC'),
            # Australia Day on a Sunday, observed on Monday (2010 and after).
            pd.Timestamp('2014-01-27', tz='UTC'),
            # Anzac Day on a Sunday, observed on Monday.
            pd.Timestamp('2010-04-26', tz='UTC'),
            # Christmas on a Sunday, Boxing Day on Monday.
            pd.Timestamp('2016-12-26', tz='UTC'),
            pd.Timestamp('2016-12-27', tz='UTC'),
            # Christmas on a Saturday, Boxing Day on Sunday.
            pd.Timestamp('2010-12-27', tz='UTC'),
            pd.Timestamp('2010-12-28', tz='UTC'),
        ]

        for session_label in expected_holidays:
            self.assertNotIn(session_label, self.calendar.all_sessions)

        expected_sessions = [
            # Anzac Day on a Saturday, does not have a make-up.
            pd.Timestamp('2015-04-27', tz='UTC'),
            # Anzac Day on a Saturday, does not have a make-up (prior
            # to 2010).
            pd.Timestamp('2004-04-26', tz='UTC'),
        ]

        for session_label in expected_sessions:
            self.assertIn(session_label, self.calendar.all_sessions)

    def test_half_days(self):
        half_days = [
            # In 2018, the last trading days before Christmas and New Year's
            # are on Mondays, so they should be half days.
            pd.Timestamp('2018-12-24', tz='Australia/Sydney'),
            pd.Timestamp('2018-12-31', tz='Australia/Sydney'),
            # In 2017, Christmas and New Year's fell on Mondays, so the last
            # trading days before them were Fridays, which should be half days.
            pd.Timestamp('2017-12-22', tz='Australia/Sydney'),
            pd.Timestamp('2017-12-29', tz='Australia/Sydney'),
            # In 2016, Christmas and New Year's fell on Sundays, so the last
            # trading days before them were Fridays, which should be half days.
            pd.Timestamp('2016-12-23', tz='Australia/Sydney'),
            pd.Timestamp('2016-12-30', tz='Australia/Sydney'),
            # 2010 is the first year we expect the half day rules to take
            # effect.
            pd.Timestamp('2010-12-24', tz='Australia/Sydney'),
            pd.Timestamp('2010-12-31', tz='Australia/Sydney'),
        ]
        full_days = [
            # In 2009 the half day rules should not be in effect yet.
            pd.Timestamp('2009-12-24', tz='Australia/Sydney'),
            pd.Timestamp('2009-12-31', tz='Australia/Sydney'),
        ]

        for half_day in half_days:
            half_day_close_time = self.calendar.next_close(half_day)
            self.assertEqual(
                half_day_close_time.tz_convert('Australia/Sydney'),
                half_day + pd.Timedelta(hours=14, minutes=10),
            )
        for full_day in full_days:
            full_day_close_time = self.calendar.next_close(full_day)
            self.assertEqual(
                full_day_close_time.tz_convert('Australia/Sydney'),
                full_day + pd.Timedelta(hours=16),
            )



from datetime import time
from itertools import chain
import pandas as pd
from pytz import timezone

from .trading_calendar import (
    TradingCalendar,
    HolidayCalendar,
    end_default,
)
from .xtks_holidays import (
    NewYearsHolidayDec31,
    NewYearsHolidayJan1,
    NewYearsHolidayJan2,
    NewYearsHolidayJan3,
    ComingOfAgeDay,
    NationalFoundationDay,
    VernalEquinoxes,
    GreeneryDayThrough2006,
    ShowaDay,
    ConstitutionMemorialDay,
    GreeneryDay2007Onwards,
    CitizensHolidayGoldenWeek,
    ChildrensDay,
    MarineDayThrough2002,
    MarineDay2003Onwards,
    MountainDay,
    AutumnalEquinoxes,
    CitizensHolidaySilverWeek,
    RespectForTheAgedDayThrough2002,
    RespectForTheAgedDay2003Onwards,
    HealthAndSportsDay,
    CultureDay,
    LaborThanksgivingDay,
    EmperorAkihitoBirthday,
    EmperorNaruhitoBirthday,
)


XTKS_START_DEFAULT = pd.Timestamp('2000-01-01', tz='UTC')


class XTKSExchangeCalendar(TradingCalendar):
    """
    Exchange calendar for the Tokyo Stock Exchange

    First session: 9:00am - 11:30am
    Lunch
    Second session: 12:30pm - 3:00pm

    NOTE: we are treating the two sessions per day as one session for now,
    because we will not be handling minutely data in the immediate future.

    Regularly-Observed Holidays (see tse_holidays.py for more info):
    - New Year's Holidays (Dec. 31 - Jan. 3)
    - Coming of Age Day (second Monday of January)
    - National Foundation Day (Feb. 11)
    - Vernal Equinox (usually Mar 20-22)
    - Greenery Day (Apr. 29 2000-2006, May 4 2007-present)
    - Showa Day (Apr. 29 2007-present)
    - Constitution Memorial Day (May 3)
    - Citizen's Holiday (May 4 2000-2006, later replaced by Greenery Day)
    - Children's Day (May 5)
    - Marine Day (July 20 2000-2002, third Monday of July 2003-present)
    - Respect for the Aged Day (Sep. 15 2000-2002, third Monday
      of Sep. 2003-present)
    - Autumnal Equinox (usually Sept. 22-24)
    - Health-Sports Day (second Monday of October)
    - Culture Day (November 3)
    - Labor Thanksgiving Day (Nov. 23)
    - Emperor's Birthday (Dec. 23)
    """
    def __init__(self, start=XTKS_START_DEFAULT, end=end_default):
        # because we are not tracking holiday info farther back than 2000,
        # make the default start date 01-01-2000
        super(XTKSExchangeCalendar, self).__init__(start=start, end=end)

    name = 'XTKS'

    tz = timezone('Asia/Tokyo')

    open_times = (
        (None, time(9, 1)),
    )

    close_times = (
        (None, time(15)),
    )

    @property
    def regular_holidays(self):
        return HolidayCalendar([
            NewYearsHolidayDec31,
            NewYearsHolidayJan1,
            NewYearsHolidayJan2,
            NewYearsHolidayJan3,
            ComingOfAgeDay,
            NationalFoundationDay,
            GreeneryDayThrough2006,
            ShowaDay,
            ConstitutionMemorialDay,
            GreeneryDay2007Onwards,
            CitizensHolidayGoldenWeek,
            ChildrensDay,
            MarineDayThrough2002,
            MarineDay2003Onwards,
            MountainDay,
            RespectForTheAgedDayThrough2002,
            RespectForTheAgedDay2003Onwards,
            HealthAndSportsDay,
            CultureDay,
            LaborThanksgivingDay,
            EmperorAkihitoBirthday,
            EmperorNaruhitoBirthday,
        ])

    @property
    def adhoc_holidays(self):
        return list(chain(
            VernalEquinoxes,
            AutumnalEquinoxes,
            CitizensHolidaySilverWeek,
        ))



#
# Copyright 2018 Quantopian, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import time
from pandas.tseries.holiday import (
    DateOffset,
    EasterMonday,
    GoodFriday,
    Holiday,
    MO,
    next_monday,
    next_monday_or_tuesday,
    previous_workday,
    weekend_to_monday,
)
from pytz import timezone

from .common_holidays import (
    new_years_day,
    anzac_day,
    christmas,
    boxing_day,
)

from .trading_calendar import (
    TradingCalendar,
    HolidayCalendar
)


# Prior to 2015, Waitangi Day and Anzac Day are not "Mondayized",
# that is, if they occur on the weekend, there is no make-up.
MONDAYIZATION_START_DATE = "2015-01-01"

# Regular Holidays
# ----------------
NewYearsDay = new_years_day(observance=next_monday)

DayAfterNewYearsDay = Holiday(
    "Day after New Year's Day",
    month=1,
    day=2,
    observance=next_monday_or_tuesday,
)

WaitangiDayNonMondayized = Holiday(
    "Waitangi Day",
    month=2,
    day=6,
    end_date=MONDAYIZATION_START_DATE,
)

WaitangiDay = Holiday(
    "Waitangi Day",
    month=2,
    day=6,
    observance=weekend_to_monday,
    start_date=MONDAYIZATION_START_DATE,
)

AnzacDayNonMondayized = anzac_day(end_date=MONDAYIZATION_START_DATE)

AnzacDay = anzac_day(
    observance=weekend_to_monday,
    start_date=MONDAYIZATION_START_DATE,
)

QueensBirthday = Holiday(
    "Queen's Birthday",
    month=6,
    day=1,
    offset=DateOffset(weekday=MO(1)),
)

LabourDay = Holiday(
    "Labour Day",
    month=10,
    day=1,
    offset=DateOffset(weekday=MO(4)),
)

Christmas = christmas(observance=next_monday)

BoxingDay = boxing_day(observance=next_monday_or_tuesday)


# Early Closes
# ------------
BusinessDayPriorToChristmasDay = Holiday(
    "Business Day prior to Christmas Day",
    month=12,
    day=25,
    observance=previous_workday,
    start_date="2011-01-01",
)

BusinessDayPriorToNewYearsDay = Holiday(
    "Business Day prior to New Year's Day",
    month=1,
    day=1,
    observance=previous_workday,
    start_date="2011-01-01",
)


class XNZEExchangeCalendar(TradingCalendar):
    """
    Exchange calendar for the New Zealand Exchange (NZX).

    Open Time: 10:00 AM, NZ
    Close Time: 4:45 PM, NZ

    Regularly-Observed Holidays:
    - New Year's Day
    - Day after New Year's Day
    - Waitangi Day
    - Good Friday
    - Easter Monday
    - Anzac Day
    - Queen's Birthday
    - Labour Day
    - Christmas
    - Boxing Day

    Early Closes:
    - Business Day prior to Christmas Day
    - Business Day prior to New Year's Day
    """
    regular_early_close = time(12, 45)

    name = 'XNZE'

    tz = timezone('NZ')

    open_times = (
        (None, time(10, 1)),
    )

    close_times = (
        (None, time(16, 45)),
    )

    @property
    def regular_holidays(self):
        return HolidayCalendar([
            NewYearsDay,
            DayAfterNewYearsDay,
            WaitangiDayNonMondayized,
            WaitangiDay,
            GoodFriday,
            EasterMonday,
            AnzacDayNonMondayized,
            AnzacDay,
            QueensBirthday,
            LabourDay,
            Christmas,
            BoxingDay,
        ])

    @property
    def special_closes(self):
        return [
            (
                self.regular_early_close,
                HolidayCalendar([
                    BusinessDayPriorToChristmasDay,
                    BusinessDayPriorToNewYearsDay
                ])
            )
        ]



from unittest import TestCase

import pandas as pd
from pandas.util.testing import assert_index_equal

from trading_calendars.always_open import AlwaysOpenCalendar
from .test_trading_calendar import ExchangeCalendarTestBase


class AlwaysOpenTestCase(ExchangeCalendarTestBase, TestCase):

    answer_key_filename = '24-7'
    calendar_class = AlwaysOpenCalendar
    start_date = pd.Timestamp('2016', tz='UTC'),
    end_date = pd.Timestamp('2016-12-31', tz='UTC'),
    MAX_SESSION_HOURS = 24
    GAPS_BETWEEN_SESSIONS = False
    HAVE_EARLY_CLOSES = False

    MINUTE_INDEX_TO_SESSION_LABELS_START = pd.Timestamp('2016-01-01', tz='UTC')
    MINUTE_INDEX_TO_SESSION_LABELS_END = pd.Timestamp('2016-04-04', tz='UTC')

    DAYLIGHT_SAVINGS_DATES = ['2016-04-05', '2016-11-01']

    @classmethod
    def setUpClass(cls):
        cls.cal = AlwaysOpenCalendar(
            start=pd.Timestamp('2016', tz='UTC'),
            end=pd.Timestamp('2016-12-31', tz='UTC'),
        )

    def test_open_every_day(self):
        cal = self.calendar

        #    February 2016
        # Su Mo Tu We Th Fr Sa
        #     1  2  3  4  5  6
        #  7  8  9 10 11 12 13
        # 14 15 16 17 18 19 20
        # 21 22 23 24 25 26 27
        # 28 29
        dates = pd.date_range('2016-02-01', '2016-02-28', tz='utc')
        cal_dates = cal.sessions_in_range(dates[0], dates[-1])
        assert_index_equal(dates, cal_dates)

    def test_open_every_minute(self):
        cal = self.calendar
        minutes = pd.date_range(
            '2016-02-01', '2016-02-28 23:59:00', freq='min', tz='utc'
        )
        cal_minutes = cal.minutes_for_sessions_in_range(
            pd.Timestamp('2016-02-01', tz='utc'),
            pd.Timestamp('2016-02-28', tz='utc'),
        )
        assert_index_equal(minutes, cal_minutes)

    def test_start_end(self):

        start = pd.Timestamp('2010-1-3', tz='UTC')
        end = pd.Timestamp('2010-1-10', tz='UTC')
        calendar = self.calendar_class(start=start, end=end)

        self.assertTrue(calendar.first_trading_session == start)
        self.assertTrue(calendar.last_trading_session == end)



from unittest import TestCase
import pandas as pd

from .test_trading_calendar import ExchangeCalendarTestBase
from trading_calendars.exchange_calendar_xwbo import XWBOExchangeCalendar


class XWBOCalendarTestCase(ExchangeCalendarTestBase, TestCase):

    answer_key_filename = 'xwbo'
    calendar_class = XWBOExchangeCalendar

    # The XWBO is open from 9:00 am to 5:30 pm.
    MAX_SESSION_HOURS = 8.5

    HAVE_EARLY_CLOSES = False

    DAYLIGHT_SAVINGS_DATES = ['2018-03-12', '2018-11-05']

    def test_normal_holidays(self):
        expected_holidays = [
            pd.Timestamp('2014-01-01', tz='UTC'),  # New Year's Day
            pd.Timestamp('2014-01-06', tz='UTC'),  # Epiphany
            pd.Timestamp('2014-04-18', tz='UTC'),  # Good Friday
            pd.Timestamp('2014-04-21', tz='UTC'),  # Easter Monday
            pd.Timestamp('2014-05-01', tz='UTC'),  # Labour Day
            pd.Timestamp('2014-05-29', tz='UTC'),  # Ascension Day
            pd.Timestamp('2014-06-09', tz='UTC'),  # Whit Monday
            pd.Timestamp('2014-06-19', tz='UTC'),  # Corpus Christi
            pd.Timestamp('2014-08-15', tz='UTC'),  # Assumption Day
            pd.Timestamp('2014-10-26', tz='UTC'),  # National Day (Weekend)
            pd.Timestamp('2015-10-26', tz='UTC'),  # National Day (Weekday)
            pd.Timestamp('2013-11-01', tz='UTC'),  # All Saints Day (Weekday)
            pd.Timestamp('2014-11-01', tz='UTC'),  # All Saints Day (Weekend)
            pd.Timestamp('2014-12-08', tz='UTC'),  # Immaculate Conception
            pd.Timestamp('2014-12-24', tz='UTC'),  # Christmas Eve
            pd.Timestamp('2014-12-25', tz='UTC'),  # Christmas Day
            pd.Timestamp('2014-12-26', tz='UTC'),  # St. Stephen's Day
            pd.Timestamp('2014-12-31', tz='UTC'),  # New Year's Eve
        ]

        for session_label in expected_holidays:
            self.assertNotIn(session_label, self.calendar.all_sessions)

    def test_holidays_fall_on_weekend(self):
        # Holidays falling on a weekend should generally not be made up
        # during the week, so test that the Fridays and Mondays surrounding
        # them are open market days.
        expected_sessions = [
            # Epiphany (January 6th) on a Saturday.
            pd.Timestamp('2018-01-05', tz='UTC'),
            pd.Timestamp('2018-01-08', tz='UTC'),
            # Assumption Day (August 15th) on a Saturday.
            pd.Timestamp('2015-08-14', tz='UTC'),
            pd.Timestamp('2015-08-17', tz='UTC'),
            # Labour Day (May 1st) on a Saturday.
            pd.Timestamp('2010-04-30', tz='UTC'),
            pd.Timestamp('2010-05-03', tz='UTC'),
            # National Day (October 26th) on a Sunday.
            pd.Timestamp('2014-10-24', tz='UTC'),
            pd.Timestamp('2014-10-27', tz='UTC'),
            # All Saints Day (November 1st) on a Sunday.
            pd.Timestamp('2015-10-30', tz='UTC'),
            pd.Timestamp('2015-11-02', tz='UTC'),
            # Immaculate Conception (December 8th) on a Saturday.
            pd.Timestamp('2018-12-07', tz='UTC'),
            pd.Timestamp('2018-12-10', tz='UTC'),
            # Christmas Eve on a Saturday and Christmas on a Sunday. This means
            # that the market should be open on the previous Friday, closed on
            # Monday for St. Stephen's Day, and open again on Tuesday.
            pd.Timestamp('2011-12-23', tz='UTC'),
            pd.Timestamp('2011-12-27', tz='UTC'),
        ]

        for session_label in expected_sessions:
            self.assertIn(session_label, self.calendar.all_sessions)

    def test_new_years_eve_falls_on_weekend(self):
        # Prior to 2016, when New Year's Eve fell on the weekend, it was
        # observed on the preceding Friday.
        expected_holidays = [
            # New Year's Eve on a Saturday, observed on Friday 12/30.
            pd.Timestamp('2011-12-30', tz='UTC'),
            # New Year's Eve on a Sunday, observed on Friday 12/29.
            pd.Timestamp('2006-12-29', tz='UTC'),
        ]

        for holiday_label in expected_holidays:
            self.assertNotIn(holiday_label, self.calendar.all_sessions)

        # In 2016 and after, it is not made up.
        expected_sessions = [
            # New Year's Eve on a Saturday, Friday 12/30 is a trading day.
            pd.Timestamp('2016-12-30', tz='UTC'),
            # New Year's Eve on a Sunday, Friday 12/29 is a trading day.
            pd.Timestamp('2017-12-29', tz='UTC'),
        ]

        for session_label in expected_sessions:
            self.assertIn(session_label, self.calendar.all_sessions)



from unittest import TestCase
import pandas as pd

from .test_trading_calendar import ExchangeCalendarTestBase
from trading_calendars.exchange_calendar_xtse import XTSEExchangeCalendar


class XTSECalendarTestCase(ExchangeCalendarTestBase, TestCase):

    answer_key_filename = 'xtse'
    calendar_class = XTSEExchangeCalendar

    MAX_SESSION_HOURS = 6.5

    def test_2012(self):
        expected_holidays_2012 = [
            pd.Timestamp("2012-01-02", tz='UTC'),  # New Year's observed
            pd.Timestamp("2012-02-20", tz='UTC'),  # Family Day
            pd.Timestamp("2012-04-06", tz='UTC'),  # Good Friday
            pd.Timestamp("2012-05-21", tz='UTC'),  # Victoria Day
            pd.Timestamp("2012-07-02", tz='UTC'),  # Canada Day
            pd.Timestamp("2012-08-06", tz='UTC'),  # Civic Holiday
            pd.Timestamp("2012-09-03", tz='UTC'),  # Labour Day
            pd.Timestamp("2012-10-08", tz='UTC'),  # Thanksgiving
            pd.Timestamp("2012-12-25", tz='UTC'),  # Christmas
            pd.Timestamp("2012-12-26", tz='UTC'),  # Boxing Day
        ]

        for session_label in expected_holidays_2012:
            self.assertNotIn(session_label, self.calendar.all_sessions)

        # early closes we expect:
        early_closes_2012 = [
            pd.Timestamp("2012-12-24", tz='UTC')
        ]

        for early_close_session_label in early_closes_2012:
            self.assertIn(early_close_session_label,
                          self.calendar.early_closes)

    def test_special_holidays(self):
        # 9/11
        # Sept 11, 12, 2001
        self.assertNotIn(pd.Period("9/11/2001"), self.calendar.all_sessions)
        self.assertNotIn(pd.Period("9/12/2001"), self.calendar.all_sessions)

    def test_new_years(self):
        """
        Check whether the TradingCalendar contains certain dates.
        """
        #     January 2012
        # Su Mo Tu We Th Fr Sa
        #  1  2  3  4  5  6  7
        #  8  9 10 11 12 13 14
        # 15 16 17 18 19 20 21
        # 22 23 24 25 26 27 28
        # 29 30 31

        start_session = pd.Timestamp("2012-01-02", tz='UTC')
        end_session = pd.Timestamp("2013-12-31", tz='UTC')
        sessions = self.calendar.sessions_in_range(start_session, end_session)

        day_after_new_years_sunday = pd.Timestamp("2012-01-02", tz='UTC')
        self.assertNotIn(
            day_after_new_years_sunday,
            sessions,
            "If NYE falls on a weekend, {0} the Monday after is a holiday."
            .format(day_after_new_years_sunday)
        )

        first_trading_day_after_new_years_sunday = pd.Timestamp("2012-01-03",
                                                                tz='UTC')
        self.assertIn(
            first_trading_day_after_new_years_sunday,
            sessions,
            "If NYE falls on a weekend, {0} the Tuesday after is the "
            "first trading day.".format(
                first_trading_day_after_new_years_sunday
            )
        )

        #     January 2013
        # Su Mo Tu We Th Fr Sa
        #        1  2  3  4  5
        #  6  7  8  9 10 11 12
        # 13 14 15 16 17 18 19
        # 20 21 22 23 24 25 26
        # 27 28 29 30 31

        new_years_day = pd.Timestamp("2013-01-01", tz='UTC')
        self.assertNotIn(
            new_years_day,
            sessions,
            "If NYE falls during the week, e.g. {0}, it is a holiday."
            .format(new_years_day)
        )

        first_trading_day_after_new_years = pd.Timestamp("2013-01-02",
                                                         tz='UTC')
        self.assertIn(
            first_trading_day_after_new_years,
            sessions,
            "If the day after NYE falls during the week, {0} is the first "
            "trading day.".format(first_trading_day_after_new_years)
        )

    def test_christmas_eve_half_day(self):
        #    December 2009
        # Su Mo Tu We Th Fr Sa
        #        1  2  3  4  5
        #  6  7  8  9 10 11 12
        # 13 14 15 16 17 18 19
        # 20 21 22 23 24 25 26
        # 27 28 29 30 31

        # Christmas eve fell on a weekday in 2009 and 2010, but
        # it is only a half day from 2010 onwards
        christmas_eve09 = pd.Timestamp('2009-12-24')
        christmas_eve09_close = self.calendar.next_close(christmas_eve09)
        self.assertEqual(
            christmas_eve09_close.tz_convert('Canada/Atlantic'),
            pd.Timestamp('2009-12-24 4:00 PM', tz='Canada/Atlantic')
        )

        christmas_eve10 = pd.Timestamp('2010-12-24')
        christmas_eve10_close = self.calendar.next_close(christmas_eve10)
        self.assertEqual(
            christmas_eve10_close.tz_convert('Canada/Atlantic'),
            pd.Timestamp('2010-12-24 1:00 PM', tz='Canada/Atlantic')
        )

        #    December 2012
        # Su Mo Tu We Th Fr Sa
        #                    1
        #  2  3  4  5  6  7  8
        #  9 10 11 12 13 14 15
        # 16 17 18 19 20 21 22
        # 23 24 25 26 27 28 29
        # 30 31

        # In 2012, 2013, 2014, and 2015, Christmas eve fell on a Monday,
        # Tuesday, Wednesday, and Thursday respectively, so it should
        # be a half day on all of those days
        for year in ['2012', '2013', '2014', '2015']:
            christmas_eve = pd.Timestamp('{}-12-24'.format(year))
            christmas_eve_close = self.calendar.next_close(christmas_eve)

            self.assertEqual(
                christmas_eve_close.tz_convert("Canada/Atlantic"),
                pd.Timestamp('{}-12-24 1:00 PM'.format(year),
                             tz="Canada/Atlantic"),
            )

    def test_christmas(self):
        #    December 2015
        # Su Mo Tu We Th Fr Sa
        #        1  2  3  4  5
        #  6  7  8  9 10 11 12
        # 13 14 15 16 17 18 19
        # 20 21 22 23 24 25 26
        # 27 28 29 30 31

        # In 2015 Christmas fell on a Friday so Boxing Day should
        # be celebrated the following Monday
        christmas = pd.Timestamp('2015-12-25', tz='UTC')
        boxing_day_observed = pd.Timestamp('2015-12-28', tz='UTC')

        self.assertNotIn(christmas, self.calendar.all_sessions)
        self.assertNotIn(boxing_day_observed, self.calendar.all_sessions)

        #    December 2010
        # Su Mo Tu We Th Fr Sa
        #           1  2  3  4
        #  5  6  7  8  9 10 11
        # 12 13 14 15 16 17 18
        # 19 20 21 22 23 24 25
        # 26 27 28 29 30 31

        # Christmas fell on a Saturday in 2010, so the following two trading
        # days should be holidays
        christmas_observed = pd.Timestamp('2016-12-26', tz='UTC')
        boxing_day_observed = pd.Timestamp('2016-12-27', tz='UTC')

        self.assertNotIn(christmas, self.calendar.all_sessions)
        self.assertNotIn(boxing_day_observed, self.calendar.all_sessions)

        #    December 2016
        # Su Mo Tu We Th Fr Sa
        #              1  2  3
        #  4  5  6  7  8  9 10
        # 11 12 13 14 15 16 17
        # 18 19 20 21 22 23 24
        # 25 26 27 28 29 30 31

        # Christmas fell on a Sunday in 2016, so the 26th and 27th should
        # be holidays
        christmas_observed = pd.Timestamp('2016-12-26', tz='UTC')
        boxing_day_observed = pd.Timestamp('2016-12-27', tz='UTC')

        self.assertNotIn(christmas_observed, self.calendar.all_sessions)
        self.assertNotIn(boxing_day_observed, self.calendar.all_sessions)

    def test_victoria_day(self):
        #       May 2015
        # Su Mo Tu We Th Fr Sa
        #                 1  2
        #  3  4  5  6  7  8  9
        # 10 11 12 13 14 15 16
        # 17 18 19 20 21 22 23
        # 24 25 26 27 28 29 30
        # 31

        # Victoria Day is never held on Monday 5/25...
        self.assertIn(
            pd.Timestamp('2015-05-25'),
            self.calendar.all_sessions,
        )

        # ...but on the Monday preceding 5/25.
        self.assertNotIn(
            pd.Timestamp('2015-05-18'),
            self.calendar.all_sessions,
        )



#
# Copyright 2018 Quantopian, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import time

from pandas.tseries.holiday import (
    Holiday,
    GoodFriday,
    Easter,
    EasterMonday,
    DateOffset,
    FR,
)
from pandas.tseries.offsets import Day
from pytz import timezone

from .common_holidays import (
    new_years_day,
    epiphany,
    maundy_thursday,
    ascension_day,
    whit_monday,
    european_labour_day,
    midsummer_eve,
    christmas_eve,
    christmas,
    boxing_day,
    new_years_eve,
)
from .trading_calendar import HolidayCalendar, TradingCalendar, WEEKDAYS

NewYearsDay = new_years_day()

DayBeforeEpiphany = Holiday(
    'Day Before Epiphany',
    month=1,
    day=5,
    days_of_week=WEEKDAYS,
)
Epiphany = epiphany()

DayBeforeLabourDay = Holiday(
    'Day Before Labour Day',
    month=4,
    day=30,
    days_of_week=WEEKDAYS,
)
LabourDay = european_labour_day()

MaundyThursday = maundy_thursday(days_of_week=WEEKDAYS)
DayBeforeAscensionDay = Holiday(
    'Day Before Ascension Day',
    month=1,
    day=1,
    offset=[Easter(), Day(38)],
)
AscensionDay = ascension_day()
WhitMonday = whit_monday(end_date='2005')

NationalDay = Holiday('Sweden National Day', month=6, day=6, start_date='2004')

MidsummerEve = midsummer_eve()

# This falls on the Friday between October 30th and November 5th.
AllSaintsEve = Holiday(
    "All Saints' Eve",
    month=10,
    day=30,
    offset=DateOffset(weekday=FR(1)),
)

ChristmasEve = christmas_eve()
Christmas = christmas()
BoxingDay = boxing_day()

NewYearsEve = new_years_eve()


class XSTOExchangeCalendar(TradingCalendar):
    """
    Calendar for the Stockholm Stock Exchange in Sweden.

    Open Time: 9:00 AM, CET (Central European Time)
    Close Time: 5:30 PM, CET (Central European Time)

    Regularly-Observed Holidays:
      - New Year's Day
      - Epiphany
      - Good Friday
      - Easter Monday
      - Labour Day
      - Ascension Day
      - National Day
      - Midsummer Eve
      - Christmas Eve
      - Christmas Day
      - Boxing Day
      - New Year's Eve

    Holidays No Longer Observed:
      - Whit Monday

    Early Closes:
      - Day before Epiphany
      - Maundy Thursday
      - Day before Labour Day
      - Day before Ascension Day
      - All Saints' Eve
    """
    name = 'XSTO'
    tz = timezone('Europe/Stockholm')
    open_times = (
        (None, time(9, 1)),
    )
    close_times = (
        (None, time(17, 30)),
    )
    regular_early_close = time(13)

    @property
    def regular_holidays(self):
        return HolidayCalendar([
            NewYearsDay,
            Epiphany,
            GoodFriday,
            EasterMonday,
            LabourDay,
            AscensionDay,
            WhitMonday,
            NationalDay,
            MidsummerEve,
            ChristmasEve,
            Christmas,
            BoxingDay,
            NewYearsEve,
        ])

    @property
    def special_closes(self):
        return [
            (
                self.regular_early_close,
                HolidayCalendar([
                    DayBeforeEpiphany,
                    MaundyThursday,
                    DayBeforeLabourDay,
                    DayBeforeAscensionDay,
                    AllSaintsEve,
                ]),
            ),
        ]



#
# Copyright 2018 Quantopian, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import time
from itertools import chain

from pandas import DatetimeIndex
from pandas.tseries.holiday import (
    GoodFriday,
    USLaborDay,
    USPresidentsDay,
    USThanksgivingDay,
)
from pytz import timezone

from .trading_calendar import TradingCalendar, HolidayCalendar
from .us_holidays import (
    USNewYearsDay,
    USMartinLutherKingJrAfter1998,
    USMemorialDay,
    USIndependenceDay,
    Christmas,
    MonTuesThursBeforeIndependenceDay,
    FridayAfterIndependenceDayExcept2013,
    USBlackFridayBefore1993,
    USBlackFridayInOrAfter1993,
    September11Closings,
    HurricaneSandyClosings,
    USNationalDaysofMourning,
    ChristmasEveBefore1993,
    ChristmasEveInOrAfter1993,
)

# Useful resources for making changes to this file:
# http://www.nyse.com/pdfs/closings.pdf
# http://www.stevemorse.org/jcal/whendid.html


class XNYSExchangeCalendar(TradingCalendar):
    """
    Exchange calendar for the New York Stock Exchange (XNYS).

    Open Time: 9:31 AM, US/Eastern
    Close Time: 4:00 PM, US/Eastern

    Regularly-Observed Holidays:
    - New Years Day (observed on monday when Jan 1 is a Sunday)
    - Martin Luther King Jr. Day (3rd Monday in January, only after 1998)
    - Washington's Birthday (aka President's Day, 3rd Monday in February)
    - Good Friday (two days before Easter Sunday)
    - Memorial Day (last Monday in May)
    - Independence Day (observed on the nearest weekday to July 4th)
    - Labor Day (first Monday in September)
    - Thanksgiving (fourth Thursday in November)
    - Christmas (observed on nearest weekday to December 25)

    NOTE: The NYSE does not observe the following US Federal Holidays:
    - Columbus Day
    - Veterans Day

    Regularly-Observed Early Closes:
    - July 3rd (Mondays, Tuesdays, and Thursdays, 1995 onward)
    - July 5th (Fridays, 1995 onward, except 2013)
    - Christmas Eve (except on Fridays, when the exchange is closed entirely)
    - Day After Thanksgiving (aka Black Friday, observed from 1992 onward)

    NOTE: Until 1993, the standard early close time for the NYSE was 2:00 PM.
    From 1993 onward, it has been 1:00 PM.

    Additional Irregularities:
    - Closed from 9/11/2001 to 9/16/2001 due to terrorist attacks in NYC.
    - Closed on 10/29/2012 and 10/30/2012 due to Hurricane Sandy.
    - Closed on 4/27/1994 due to Richard Nixon's death.
    - Closed on 6/11/2004 due to Ronald Reagan's death.
    - Closed on 1/2/2007 due to Gerald Ford's death.
    - Closed at 1:00 PM on Wednesday, July 3rd, 2013
    - Closed at 1:00 PM on Friday, December 31, 1999
    - Closed at 1:00 PM on Friday, December 26, 1997
    - Closed at 1:00 PM on Friday, December 26, 2003

    NOTE: The exchange was **not** closed early on Friday December 26, 2008,
    nor was it closed on Friday December 26, 2014. The next Thursday Christmas
    will be in 2025.  If someone is still maintaining this code in 2025, then
    we've done alright...and we should check if it's a half day.
    """

    regular_early_close = time(13)

    name = 'XNYS'

    tz = timezone('US/Eastern')

    open_times = (
        (None, time(9, 31)),
    )

    close_times = (
        (None, time(16)),
    )

    @property
    def regular_holidays(self):
        return HolidayCalendar([
            USNewYearsDay,
            USMartinLutherKingJrAfter1998,
            USPresidentsDay,
            GoodFriday,
            USMemorialDay,
            USIndependenceDay,
            USLaborDay,
            USThanksgivingDay,
            Christmas,
        ])

    @property
    def adhoc_holidays(self):
        return list(chain(
            September11Closings,
            HurricaneSandyClosings,
            USNationalDaysofMourning,
        ))

    @property
    def special_closes(self):
        return [
            (self.regular_early_close, HolidayCalendar([
                MonTuesThursBeforeIndependenceDay,
                FridayAfterIndependenceDayExcept2013,
                USBlackFridayInOrAfter1993,
                ChristmasEveInOrAfter1993
            ])),
            (time(14), HolidayCalendar([
                ChristmasEveBefore1993,
                USBlackFridayBefore1993,
            ])),
        ]

    @property
    def special_closes_adhoc(self):
        return [
            (
                self.regular_early_close,
                DatetimeIndex(
                    [
                        '1997-12-26',
                        '1999-12-31',
                        '2003-12-26',
                        '2013-07-03',
                    ],
                    tz='UTC',
                )
            )
        ]



from unittest import TestCase
import pandas as pd

from .test_trading_calendar import ExchangeCalendarTestBase
from trading_calendars.exchange_calendar_xnys import XNYSExchangeCalendar


class XNYSCalendarTestCase(ExchangeCalendarTestBase, TestCase):

    answer_key_filename = 'xnys'
    calendar_class = XNYSExchangeCalendar

    MAX_SESSION_HOURS = 6.5

    def test_2012(self):
        # holidays we expect:
        holidays_2012 = [
            pd.Timestamp("2012-01-02", tz='UTC'),
            pd.Timestamp("2012-01-16", tz='UTC'),
            pd.Timestamp("2012-02-20", tz='UTC'),
            pd.Timestamp("2012-04-06", tz='UTC'),
            pd.Timestamp("2012-05-28", tz='UTC'),
            pd.Timestamp("2012-07-04", tz='UTC'),
            pd.Timestamp("2012-09-03", tz='UTC'),
            pd.Timestamp("2012-11-22", tz='UTC'),
            pd.Timestamp("2012-12-25", tz='UTC')
        ]

        for session_label in holidays_2012:
            self.assertNotIn(session_label, self.calendar.all_sessions)

        # early closes we expect:
        early_closes_2012 = [
            pd.Timestamp("2012-07-03", tz='UTC'),
            pd.Timestamp("2012-11-23", tz='UTC'),
            pd.Timestamp("2012-12-24", tz='UTC')
        ]

        for early_close_session_label in early_closes_2012:
            self.assertIn(early_close_session_label,
                          self.calendar.early_closes)

    def test_special_holidays(self):
        # 9/11
        # Sept 11, 12, 13, 14 2001
        self.assertNotIn(pd.Period("9/11/2001"), self.calendar.all_sessions)
        self.assertNotIn(pd.Period("9/12/2001"), self.calendar.all_sessions)
        self.assertNotIn(pd.Period("9/13/2001"), self.calendar.all_sessions)
        self.assertNotIn(pd.Period("9/14/2001"), self.calendar.all_sessions)

        # Hurricane Sandy
        # Oct 29, 30 2012
        self.assertNotIn(pd.Period("10/29/2012"), self.calendar.all_sessions)
        self.assertNotIn(pd.Period("10/30/2012"), self.calendar.all_sessions)

        # various national days of mourning
        # Gerald Ford - 1/2/2007
        self.assertNotIn(pd.Period("1/2/2007"), self.calendar.all_sessions)

        # Ronald Reagan - 6/11/2004
        self.assertNotIn(pd.Period("6/11/2004"), self.calendar.all_sessions)

        # Richard Nixon - 4/27/1994
        self.assertNotIn(pd.Period("4/27/1994"), self.calendar.all_sessions)

    def test_new_years(self):
        """
        Check whether the TradingCalendar contains certain dates.
        """
        #     January 2012
        # Su Mo Tu We Th Fr Sa
        #  1  2  3  4  5  6  7
        #  8  9 10 11 12 13 14
        # 15 16 17 18 19 20 21
        # 22 23 24 25 26 27 28
        # 29 30 31

        start_session = pd.Timestamp("2012-01-02", tz='UTC')
        end_session = pd.Timestamp("2013-12-31", tz='UTC')
        sessions = self.calendar.sessions_in_range(start_session, end_session)

        day_after_new_years_sunday = pd.Timestamp("2012-01-02",
                                                  tz='UTC')
        self.assertNotIn(day_after_new_years_sunday, sessions,
                         """
 If NYE falls on a weekend, {0} the Monday after is a holiday.
 """.strip().format(day_after_new_years_sunday)
        )

        first_trading_day_after_new_years_sunday = pd.Timestamp("2012-01-03",
                                                                tz='UTC')
        self.assertIn(first_trading_day_after_new_years_sunday, sessions,
                      """
 If NYE falls on a weekend, {0} the Tuesday after is the first trading day.
 """.strip().format(first_trading_day_after_new_years_sunday)
        )

        #     January 2013
        # Su Mo Tu We Th Fr Sa
        #        1  2  3  4  5
        #  6  7  8  9 10 11 12
        # 13 14 15 16 17 18 19
        # 20 21 22 23 24 25 26
        # 27 28 29 30 31

        new_years_day = pd.Timestamp("2013-01-01", tz='UTC')
        self.assertNotIn(new_years_day, sessions,
                         """
 If NYE falls during the week, e.g. {0}, it is a holiday.
 """.strip().format(new_years_day)
        )

        first_trading_day_after_new_years = pd.Timestamp("2013-01-02",
                                                         tz='UTC')
        self.assertIn(first_trading_day_after_new_years, sessions,
                      """
 If the day after NYE falls during the week, {0} \
 is the first trading day.
 """.strip().format(first_trading_day_after_new_years)
        )

    def test_thanksgiving(self):
        """
        Check TradingCalendar Thanksgiving dates.
        """
        #     November 2005
        # Su Mo Tu We Th Fr Sa
        #        1  2  3  4  5
        #  6  7  8  9 10 11 12
        # 13 14 15 16 17 18 19
        # 20 21 22 23 24 25 26
        # 27 28 29 30

        start_session_label = pd.Timestamp('2005-01-01', tz='UTC')
        end_session_label = pd.Timestamp('2012-12-31', tz='UTC')
        sessions = self.calendar.sessions_in_range(start_session_label,
                                                   end_session_label)

        thanksgiving_with_four_weeks = pd.Timestamp("2005-11-24", tz='UTC')

        self.assertNotIn(thanksgiving_with_four_weeks, sessions,
                         """
 If Nov has 4 Thursdays, {0} Thanksgiving is the last Thursday.
 """.strip().format(thanksgiving_with_four_weeks)
        )

        #     November 2006
        # Su Mo Tu We Th Fr Sa
        #           1  2  3  4
        #  5  6  7  8  9 10 11
        # 12 13 14 15 16 17 18
        # 19 20 21 22 23 24 25
        # 26 27 28 29 30
        thanksgiving_with_five_weeks = pd.Timestamp("2006-11-23", tz='UTC')

        self.assertNotIn(thanksgiving_with_five_weeks, sessions,
                         """
 If Nov has 5 Thursdays, {0} Thanksgiving is not the last week.
 """.strip().format(thanksgiving_with_five_weeks)
        )

        first_trading_day_after_new_years_sunday = pd.Timestamp("2012-01-03",
                                                                tz='UTC')

        self.assertIn(first_trading_day_after_new_years_sunday, sessions,
                      """
 If NYE falls on a weekend, {0} the Tuesday after is the first trading day.
 """.strip().format(first_trading_day_after_new_years_sunday)
        )

    def test_day_after_thanksgiving(self):
        #    November 2012
        # Su Mo Tu We Th Fr Sa
        #              1  2  3
        #  4  5  6  7  8  9 10
        # 11 12 13 14 15 16 17
        # 18 19 20 21 22 23 24
        # 25 26 27 28 29 30
        fourth_friday_open = pd.Timestamp('11/23/2012 11:00AM', tz='EST')
        fourth_friday = pd.Timestamp('11/23/2012 3:00PM', tz='EST')
        self.assertTrue(self.calendar.is_open_on_minute(fourth_friday_open))
        self.assertFalse(self.calendar.is_open_on_minute(fourth_friday))

        #    November 2013
        # Su Mo Tu We Th Fr Sa
        #                 1  2
        #  3  4  5  6  7  8  9
        # 10 11 12 13 14 15 16
        # 17 18 19 20 21 22 23
        # 24 25 26 27 28 29 30
        fifth_friday_open = pd.Timestamp('11/29/2013 11:00AM', tz='EST')
        fifth_friday = pd.Timestamp('11/29/2013 3:00PM', tz='EST')
        self.assertTrue(self.calendar.is_open_on_minute(fifth_friday_open))
        self.assertFalse(self.calendar.is_open_on_minute(fifth_friday))

    def test_early_close_independence_day_thursday(self):
        """
        Until 2013, the market closed early the Friday after an
        Independence Day on Thursday.  Since then, the early close is on
        Wednesday.
        """
        #      July 2002
        # Su Mo Tu We Th Fr Sa
        #     1  2  3  4  5  6
        #  7  8  9 10 11 12 13
        # 14 15 16 17 18 19 20
        # 21 22 23 24 25 26 27
        # 28 29 30 31
        wednesday_before = pd.Timestamp('7/3/2002 3:00PM', tz='EST')
        friday_after_open = pd.Timestamp('7/5/2002 11:00AM', tz='EST')
        friday_after = pd.Timestamp('7/5/2002 3:00PM', tz='EST')
        self.assertTrue(self.calendar.is_open_on_minute(wednesday_before))
        self.assertTrue(self.calendar.is_open_on_minute(friday_after_open))
        self.assertFalse(self.calendar.is_open_on_minute(friday_after))

        #      July 2013
        # Su Mo Tu We Th Fr Sa
        #     1  2  3  4  5  6
        #  7  8  9 10 11 12 13
        # 14 15 16 17 18 19 20
        # 21 22 23 24 25 26 27
        # 28 29 30 31
        wednesday_before = pd.Timestamp('7/3/2013 3:00PM', tz='EST')
        friday_after_open = pd.Timestamp('7/5/2013 11:00AM', tz='EST')
        friday_after = pd.Timestamp('7/5/2013 3:00PM', tz='EST')
        self.assertFalse(self.calendar.is_open_on_minute(wednesday_before))
        self.assertTrue(self.calendar.is_open_on_minute(friday_after_open))
        self.assertTrue(self.calendar.is_open_on_minute(friday_after))



from unittest import TestCase
import pandas as pd

from .test_trading_calendar import ExchangeCalendarTestBase
from trading_calendars.exchange_calendar_xtks import XTKSExchangeCalendar
from trading_calendars.xtks_holidays import (
    AutumnalEquinoxes,
    ChildrensDay,
    CitizensHolidaySilverWeek,
    ConstitutionMemorialDay,
    EmperorAkihitoBirthday,
    GreeneryDay2007Onwards,
    RespectForTheAgedDay2003Onwards
)
from trading_calendars.trading_calendar import WEDNESDAY, SUNDAY


class XTKSCalendarTestCase(ExchangeCalendarTestBase, TestCase):

    answer_key_filename = 'xtks'
    calendar_class = XTKSExchangeCalendar

    MAX_SESSION_HOURS = 6
    HAVE_EARLY_CLOSES = False

    def test_2012(self):
        expected_holidays_2012 = [
            pd.Timestamp("2012-01-01", tz='UTC'),  # New Year's holiday
            pd.Timestamp("2012-01-02", tz='UTC'),  # New Year's holiday
            pd.Timestamp("2012-01-03", tz='UTC'),  # New Year's holiday
            pd.Timestamp("2012-01-09", tz='UTC'),  # Coming of Age Day
            # National Foundation Day was on a Saturday so it is ignored
            pd.Timestamp("2012-03-20", tz='UTC'),  # Vernal Equinox
            pd.Timestamp("2012-04-30", tz='UTC'),  # Showa Day Observed
            pd.Timestamp("2012-05-03", tz='UTC'),  # Constitution Memorial Day
            pd.Timestamp("2012-05-04", tz='UTC'),  # Greenery Day
            # Children's Day was on a Saturday so it is ignored
            pd.Timestamp("2012-07-16", tz='UTC'),  # Marine Day
            pd.Timestamp("2012-09-17", tz='UTC'),  # Respect for the Aged Day
            # The Autumnal Equinox was on a Saturday so it is ignored
            pd.Timestamp("2012-10-08", tz='UTC'),  # Health and Sports Day
            # Culture Day was on a Saturday so it is ignored
            pd.Timestamp("2012-11-23", tz='UTC'),  # Labor Thanksgiving Day
            pd.Timestamp("2012-12-24", tz='UTC'),  # Emperor Birthday Observed
            pd.Timestamp("2012-12-31", tz='UTC'),  # New Year's holiday
        ]

        for session_label in expected_holidays_2012:
            self.assertNotIn(session_label, self.calendar.all_sessions)

    def test_golden_week(self):
        # from 2000 to 2006 May 4 was an unnamed citizen's holiday because
        # it was between Constitution Memorial Day and Children's Day
        consitution_memorial_days = ConstitutionMemorialDay.dates(
            '2000-01-01', '2007-01-01'
        )
        childrens_days = ChildrensDay.dates(
            '2000-01-01', '2007-01-01'
        )

        for cm_day, childrens_day in zip(consitution_memorial_days,
                                         childrens_days):

            # if there is only one day between Constitution Memorial
            # Day and Children's Day, that day should be a holiday
            if childrens_day - cm_day != pd.Timedelta(days=2):
                continue

            citizens_holiday = cm_day + pd.Timedelta(days=1)

            self.assertNotIn(cm_day, self.calendar.all_sessions)
            self.assertNotIn(citizens_holiday, self.calendar.all_sessions)
            self.assertNotIn(childrens_day, self.calendar.all_sessions)

        # from 2007 onwards, Greenery Day was moved to May 4
        consitution_memorial_days = ConstitutionMemorialDay.dates(
            '2007-01-01', '2019-01-01'
        )
        greenery_days = GreeneryDay2007Onwards.dates(
            '2007-01-01', '2019-01-01'
        )
        childrens_days = ChildrensDay.dates(
            '2007-01-01', '2019-01-01'
        )

        # In 2008, Greenery Day is on a Sunday, and Children's Day
        # is on a Monday, so Greenery Day should be observed on Tuesday
        #       May 2008
        # Su Mo Tu We Th Fr Sa
        #  4  5  6  7  8  9 10
        self.assertIn(pd.Timestamp('2008-05-05'), childrens_days)
        self.assertIn(pd.Timestamp('2008-05-06'), greenery_days)

        # In 2009, Consitution Memorial Day should be observed on Wednesday,
        # since it is the next weekday that is not a holiday
        #       May 2009
        # Su Mo Tu We Th Fr Sa
        #                 1  2
        #  3  4  5  6  7  8  9
        self.assertIn(pd.Timestamp('2009-05-04'), greenery_days)
        self.assertIn(pd.Timestamp('2009-05-05'), childrens_days)
        self.assertIn(pd.Timestamp('2009-05-06'), consitution_memorial_days)

        # In 2012, Children's Day should not be observed because it falls
        # on a Saturday
        #       May 2012
        # Su Mo Tu We Th Fr Sa
        #        1  2  3  4  5
        #  6  7  8  9 10 11 12
        self.assertIn(pd.Timestamp('2012-05-03'), consitution_memorial_days)
        self.assertIn(pd.Timestamp('2012-05-04'), greenery_days)

        # In 2013, May 3 and 6 should be a holiday
        #       May 2013
        # Su Mo Tu We Th Fr Sa
        #           1  2  3  4
        #  5  6  7  8  9 10 11
        self.assertIn(pd.Timestamp('2013-05-03'), consitution_memorial_days)
        self.assertIn(pd.Timestamp('2013-05-06'), childrens_days)

    def test_silver_week(self):
        def day_before(dt):
            return dt - pd.Timedelta(days=1)

        # Make sure that every Tuesday between Respect for the Aged Day and
        # the Autumnal Equinox is also a holiday
        silver_week_citizens_holidays = []
        for equinox in AutumnalEquinoxes:
            if equinox.dayofweek == WEDNESDAY:
                silver_week_citizens_holidays.append(day_before(equinox))

        expected_silver_week_holidays = CitizensHolidaySilverWeek
        self.assertEqual(silver_week_citizens_holidays,
                         expected_silver_week_holidays)

        respect_for_the_aged_days = RespectForTheAgedDay2003Onwards.dates(
            '2003-01-01',
            AutumnalEquinoxes[-1]
        )

        for citizens_holiday in silver_week_citizens_holidays:
            self.assertNotIn(citizens_holiday, self.calendar.all_sessions)
            # the day before the citizen's holiday should be Respect
            # for the Aged Day
            respect_for_the_aged_day = day_before(citizens_holiday)
            self.assertNotIn(respect_for_the_aged_day,
                             self.calendar.all_sessions)
            self.assertIn(respect_for_the_aged_day, respect_for_the_aged_days)

    def test_emperors_birthday(self):

        # The Emperor's birthday should be celebrated every year except
        # for 2019
        expected_birthdays = EmperorAkihitoBirthday.dates(
            '1990-01-01', '2020-01-01'
        )

        for year in range(1990, 2019):
            birthday = pd.Timestamp('{}-12-23'.format(year))
            if birthday.dayofweek == SUNDAY:
                birthday += pd.Timedelta(days=1)

            self.assertIn(birthday, expected_birthdays)

        self.assertNotIn(pd.Timestamp('2019-12-23'), expected_birthdays)

    def test_mountain_day(self):
        # Mountain Day was not celebrated prior to 2016.
        self.assertIn(pd.Timestamp('2015-08-11'), self.calendar.all_sessions)

        # First celebrated and observed in 2016.
        self.assertNotIn(
            pd.Timestamp('2016-08-11'),
            self.calendar.all_sessions,
        )

        #     August 2019
        # Su Mo Tu We Th Fr Sa
        #              1  2  3
        #  4  5  6  7  8  9 10
        # 11 12 13 14 15 16 17
        # 18 19 20 21 22 23 24
        # 25 26 27 28 29 30 31
        #
        # Falls on a Sunday in 2019, so it is observed on Monday.
        self.assertNotIn(
            pd.Timestamp('2019-08-12'),
            self.calendar.all_sessions,
        )



#
# Copyright 2018 Quantopian, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import time
from itertools import chain

import numpy as np
import pandas as pd
from pandas.tseries.holiday import (
    EasterMonday,
    GoodFriday,
    Holiday,
    sunday_to_monday,
)
from pytz import timezone
import toolz

from .trading_calendar import (
    TradingCalendar,
    HolidayCalendar,
    MONDAY,
    TUESDAY,
    WEDNESDAY,
    THURSDAY,
    FRIDAY,
    SATURDAY,
    SUNDAY,
)
from .common_holidays import (
    boxing_day,
    chinese_buddhas_birthday_dates,
    chinese_lunar_new_year_dates,
    christmas,
    christmas_eve,
    day_after_mid_autumn_festival_dates,
    double_ninth_festival_dates,
    dragon_boat_festival_dates,
    chinese_national_day,
    new_years_day,
    new_years_eve,
    qingming_festival_dates,
    weekend_christmas,
)
from .utils.pandas_utils import vectorized_sunday_to_monday

# Useful resources for making changes to this file:
# # /etc/lunisolar
# http://www.math.nus.edu.sg/aslaksen/calendar/cal.pdf
# https://www.hko.gov.hk/gts/time/calendarinfo.htm
#   - the almanacs on this page are also useful

weekdays = (MONDAY, TUESDAY, WEDNESDAY, THURSDAY, FRIDAY)


labor_day = Holiday(
    'Labor Day',
    month=5,
    day=1,
    observance=sunday_to_monday,
)

establishment_day = Holiday(
    'Hong Kong Special Administrative Region Establishment Day',
    month=7,
    day=1,
    observance=sunday_to_monday,
)


def boxing_day_obs(dt):
    if dt.weekday in (MONDAY, TUESDAY):
        return dt + pd.Timedelta(days=1)
    return dt


class XHKGExchangeCalendar(TradingCalendar):
    """
    Exchange calendar for the Hong Kong Stock Exchange (XHKG).

    Open Time: 9:31 AM, Asia/Hong_Kong
    Close Time: 4:00 PM, Asia/Hong_Kong

    Regularly-Observed Holidays:
    - New Years Day (observed on monday when Jan 1 is a Sunday)
    - Lunar New Year and the following 2 days. If the 3rd day of the lunar year
      is a Sunday, then the next Monday is a holiday.
    - Ching Ming Festival
    - Good Friday
    - Easter Monday
    - Buddhas Birthday
    - Dragon Boat Festival
    - Chinese National Day (observed on monday when Oct 1 is a Sunday)
    - Day Following Mid-Autumn Festival
    - Chung Yeung Festival
    - Christmas (observed on nearest weekday to December 25)
    - Day after Christmas is observed

    Regularly-Observed Early Closes:
    - Lunar New Year's Eve
    - Christmas Eve
    - New Year's Eve


    Additional Irregularities:
    - Closes frequently for severe weather.
    """
    name = 'XHKG'
    tz = timezone('Asia/Hong_Kong')

    open_times = (
        (None, time(10, 1)),
        (pd.Timestamp('2011-03-07'), time(9, 31)),
    )
    close_times = (
        (None, time(16)),
    )
    regular_early_close_times = (
        (None, time(12, 30)),
        (pd.Timestamp('2011-03-07'), time(12, 00)),
    )

    def __init__(self, *args, **kwargs):
        super(XHKGExchangeCalendar, self).__init__(*args, **kwargs)

        lunisolar_holidays = (
            chinese_buddhas_birthday_dates,
            chinese_lunar_new_year_dates,
            day_after_mid_autumn_festival_dates,
            double_ninth_festival_dates,
            dragon_boat_festival_dates,
            qingming_festival_dates,
        )
        earliest_precomputed_year = max(map(np.min, lunisolar_holidays)).year
        if earliest_precomputed_year > self.first_trading_session.year:
            raise ValueError(
                'the lunisolar holidays have only been computed back to {},'
                ' cannot instantiate the XHKG calendar back to {}'.format(
                    earliest_precomputed_year,
                    self.first_trading_session.year,
                ),
            )

        latest_precomputed_year = min(map(np.max, lunisolar_holidays)).year
        if latest_precomputed_year < self.last_trading_session.year:
            raise ValueError(
                'the lunisolar holidays have only been computed through {},'
                ' cannot instantiate the XHKG calendar in {}'.format(
                    latest_precomputed_year,
                    self.last_trading_session.year,
                ),
            )

    @property
    def regular_holidays(self):
        return HolidayCalendar([
            new_years_day(observance=sunday_to_monday),
            GoodFriday,
            EasterMonday,
            labor_day,
            establishment_day,
            chinese_national_day(observance=sunday_to_monday),
            christmas(),
            weekend_christmas(),
            boxing_day(observance=boxing_day_obs)
        ])

    @property
    def adhoc_holidays(self):
        lunar_new_years_eve = (
            chinese_lunar_new_year_dates - pd.Timedelta(days=1)
        )[
            (chinese_lunar_new_year_dates.weekday == SATURDAY) &
            (chinese_lunar_new_year_dates.year < 2013)
        ]

        lunar_new_year_2 = chinese_lunar_new_year_dates + pd.Timedelta(days=1)
        lunar_new_year_3 = chinese_lunar_new_year_dates + pd.Timedelta(days=2)
        lunar_new_year_4 = (
            chinese_lunar_new_year_dates + pd.Timedelta(days=3)
        )[
            # According to the new arrangement under the General Holidays and
            # Employment Legislation (Substitution of Holidays)(Amendment)
            # Ordinance 2011, when either Lunar New Year's Day, the second day
            # of Lunar New Year or the third day of Lunar New Year falls on a
            # Sunday, the fourth day of Lunar New Year is designated as a
            # statutory and general holiday in substitution.
            (
                (chinese_lunar_new_year_dates.weekday == SUNDAY) |
                (lunar_new_year_2.weekday == SUNDAY) |
                (lunar_new_year_3.weekday == SUNDAY)
            ) &
            (chinese_lunar_new_year_dates.year >= 2013)
        ]

        qingming_festival = vectorized_sunday_to_monday(
            qingming_festival_dates,
        ).values
        years = qingming_festival.astype('M8[Y]')
        easter_monday = EasterMonday.dates(years[0], years[-1] + 1)
        # qingming gets observed one day later if easter monday is on the same
        # day
        qingming_festival[qingming_festival == easter_monday] += (
            np.timedelta64(1, 'D')
        )

        # if the day after the mid autumn festival is October first, which
        # conflicts with national day, then national day is observed on the
        # second, though we don't encode that in the regular holiday, so
        # instead we pretend that the mid autumn festival would be delayed.
        mid_autumn_festival = day_after_mid_autumn_festival_dates.values
        mid_autumn_festival[
            (day_after_mid_autumn_festival_dates.month == 10) &
            (day_after_mid_autumn_festival_dates.day == 1)
        ] += np.timedelta64(1, 'D')

        return list(chain(
            lunar_new_years_eve,
            chinese_lunar_new_year_dates,
            lunar_new_year_2,
            lunar_new_year_3,
            lunar_new_year_4,

            qingming_festival,
            vectorized_sunday_to_monday(chinese_buddhas_birthday_dates),
            vectorized_sunday_to_monday(dragon_boat_festival_dates),
            mid_autumn_festival,
            vectorized_sunday_to_monday(double_ninth_festival_dates),

            # severe weather closure (typhoons)
            [
                '2008-08-06',
                '2008-08-22',
                '2011-09-29',
                '2013-08-14',
                '2016-08-02',
                '2016-10-21',
                '2017-08-23',
            ],

            # special holiday:
            # https://www.info.gov.hk/gia/general/201507/09/P201507080716.htm
            ['2015-09-03'],
        ))

    @property
    def special_closes(self):
        return [
            (
                time,
                HolidayCalendar([
                    new_years_eve(
                        start_date=start,
                        end_date=end,
                        days_of_week=weekdays,
                    ),
                    christmas_eve(
                        start_date=start,
                        end_date=end,
                        days_of_week=weekdays
                    ),
                ]),
            )
            for (start, time), (end, _) in toolz.sliding_window(
                2,
                toolz.concatv(self.regular_early_close_times, [(None, None)]),
            )
        ]

    @property
    def special_closes_adhoc(self):
        lunar_new_years_eve = (
            chinese_lunar_new_year_dates - pd.Timedelta(days=1)
        )[
            np.in1d(
                chinese_lunar_new_year_dates.weekday,
                [TUESDAY, WEDNESDAY, THURSDAY, FRIDAY, SATURDAY],
            ) & (chinese_lunar_new_year_dates.year >= 2013)
        ].values

        def selection(arr, start, end):
            predicates = []
            if start is not None:
                predicates.append(start.asm8 <= arr)
            if end is not None:
                predicates.append(arr < end.asm8)

            if not predicates:
                return arr

            return arr[np.all(predicates, axis=0)]

        return [
            (time, selection(lunar_new_years_eve, start, end))
            for (start, time), (end, _) in toolz.sliding_window(
                2,
                toolz.concatv(self.regular_early_close_times, [(None, None)]),
            )
        ]



from unittest import TestCase
import pandas as pd

from .test_trading_calendar import ExchangeCalendarTestBase
from trading_calendars.exchange_calendar_xdub import XDUBExchangeCalendar


class XDUBCalendarTestCase(ExchangeCalendarTestBase, TestCase):

    answer_key_filename = 'xdub'
    calendar_class = XDUBExchangeCalendar

    # The XDUB is open from 8:00 am to 4:28 pm.
    MAX_SESSION_HOURS = 8.467

    def test_normal_year(self):
        all_sessions = self.calendar.all_sessions

        expected_holidays = [
            pd.Timestamp('2018-01-01', tz='UTC'),  # New Year's Day
            pd.Timestamp('2018-03-30', tz='UTC'),  # Good Friday
            pd.Timestamp('2018-04-02', tz='UTC'),  # Easter Monday
            pd.Timestamp('2018-05-07', tz='UTC'),  # May Bank Holiday
            pd.Timestamp('2018-06-04', tz='UTC'),  # June Bank Holiday
            pd.Timestamp('2018-12-25', tz='UTC'),  # Christmas Day
            pd.Timestamp('2018-12-26', tz='UTC'),  # Boxing Day
        ]

        for session_label in expected_holidays:
            self.assertNotIn(session_label, all_sessions)

    def test_holidays_fall_on_weekend(self):
        all_sessions = self.calendar.all_sessions

        # Holidays falling on a weekend should be made up on the next trading
        # day.
        expected_holidays = [
            # In 2017 New Year's Day fell on a Sunday, so the following Monday
            # should be a holiday.
            pd.Timestamp('2017-01-02', tz='UTC'),

            # In 2010 Christmas fell on a Saturday, meaning Boxing Day fell on
            # a Sunday. The following Monday and Tuesday should both be
            # holidays.
            pd.Timestamp('2010-12-27', tz='UTC'),
            pd.Timestamp('2010-12-28', tz='UTC'),

            # In 2016 Christmas fell on a Sunday, but again the following
            # Monday and Tuesday should both be holidays.
            pd.Timestamp('2016-12-26', tz='UTC'),
            pd.Timestamp('2016-12-27', tz='UTC'),
        ]

        for session_label in expected_holidays:
            self.assertNotIn(session_label, all_sessions)

    def test_old_holidays(self):
        """
        Test the before and after of holidays that are no longer observed.
        """
        all_sessions = self.calendar.all_sessions

        expected_holidays = [
            pd.Timestamp('1996-03-18', tz='UTC'),  # St. Patrick's Day Observed
            pd.Timestamp('2000-03-17', tz='UTC'),  # St. Patrick's Day
            pd.Timestamp('2009-05-01', tz='UTC'),  # Labour Day
        ]

        for session_label in expected_holidays:
            self.assertNotIn(session_label, all_sessions)

        expected_sessions = [
            pd.Timestamp('2001-03-19', tz='UTC'),  # St. Patrick's Day Observed
            pd.Timestamp('2003-03-17', tz='UTC'),  # St. Patrick's Day
            pd.Timestamp('2012-05-01', tz='UTC'),  # Labour Day
        ]

        for session_label in expected_sessions:
            self.assertIn(session_label, all_sessions)

    def test_ad_hoc_holidays(self):
        # March 2, 2018 was closed due to sever weather.
        self.assertNotIn(
            pd.Timestamp('2018-03-02', tz='UTC'),
            self.calendar.all_sessions,
        )

    def test_early_closes(self):
        # The session label and close time for expected early closes.
        expected_early_closes = [
            # Christmas Eve on a weekday.
            (
                pd.Timestamp('2010-12-24', tz='UTC'),
                pd.Timestamp('2010-12-24 12:28', tz='Europe/Dublin'),
            ),
            (
                pd.Timestamp('2018-12-24', tz='UTC'),
                pd.Timestamp('2018-12-24 12:28', tz='Europe/Dublin'),
            ),
            # If Christmas Eve falls on a weekend the last trading day before
            # Christmas should be a trading day.
            (
                pd.Timestamp('2017-12-22', tz='UTC'),
                pd.Timestamp('2017-12-22 12:28', tz='Europe/Dublin'),
            ),
            # New Year's Eve on a weekday.
            (
                pd.Timestamp('2010-12-31', tz='UTC'),
                pd.Timestamp('2010-12-31 12:28', tz='Europe/Dublin'),
            ),
            (
                pd.Timestamp('2018-12-31', tz='UTC'),
                pd.Timestamp('2018-12-31 12:28', tz='Europe/Dublin'),
            ),
            # If New Year's Eve falls on a weekend the last trading day of the
            # year should be a half day.
            (
                pd.Timestamp('2017-12-29', tz='UTC'),
                pd.Timestamp('2017-12-29 12:28', tz='Europe/Dublin'),
            ),
            # March 1, 2018 was a half day due to severe weather.
            (
                pd.Timestamp('2018-03-01', tz='UTC'),
                pd.Timestamp('2018-03-01 12:28', tz='Europe/Dublin'),
            ),
        ]

        for session, expected_close in expected_early_closes:
            self.assertEqual(
                self.calendar.session_close(session),
                expected_close,
            )

        expected_full_days = [
            # Prior to 2010 Christmas Eve and New Year's Eve were full days.
            (
                pd.Timestamp('2009-12-24', tz='UTC'),
                pd.Timestamp('2009-12-24 16:28', tz='Europe/Dublin'),
            ),
            (
                pd.Timestamp('2009-12-31', tz='UTC'),
                pd.Timestamp('2009-12-31 16:28', tz='Europe/Dublin'),
            ),
            # March 1st on any other year should be a normal day.
            (
                pd.Timestamp('2017-03-01', tz='UTC'),
                pd.Timestamp('2017-03-01 16:28', tz='Europe/Dublin'),
            ),
        ]

        for session, expected_close in expected_full_days:
            self.assertEqual(
                self.calendar.session_close(session),
                expected_close,
            )



from datetime import time

from trading_calendars import TradingCalendar


class AlwaysOpenCalendar(TradingCalendar):
    """A TradingCalendar for an exchange that's open every minute of every day.
    """
    name = '24/7'
    tz = 'UTC'
    weekmask = '1111111'
    open_times = (
        (None, time(0)),
    )
    close_times = (
        (None, time(23, 59)),
    )



from unittest import TestCase

import pandas as pd

from .test_trading_calendar import ExchangeCalendarTestBase
from trading_calendars.exchange_calendar_xnze import XNZEExchangeCalendar


class XNZECalendarTestCase(ExchangeCalendarTestBase, TestCase):

    answer_key_filename = 'xnze'
    calendar_class = XNZEExchangeCalendar

    MINUTE_INDEX_TO_SESSION_LABELS_START = pd.Timestamp('2011-01-05', tz='UTC')
    MINUTE_INDEX_TO_SESSION_LABELS_END = pd.Timestamp('2011-04-05', tz='UTC')

    TEST_START_END_EXPECTED_FIRST = pd.Timestamp('2010-1-5', tz='UTC')

    # The New Zealand Exchange is open from 10:00 am to 4:45 pm.
    MAX_SESSION_HOURS = 6.75

    def test_normal_year(self):
        expected_holidays_2014 = [
            pd.Timestamp('2014-01-01', tz='UTC'),  # New Year's Day
            pd.Timestamp('2014-01-02', tz='UTC'),  # Day after New Year's Day
            pd.Timestamp('2014-02-06', tz='UTC'),  # Waitangi Day
            pd.Timestamp('2014-04-18', tz='UTC'),  # Good Friday
            pd.Timestamp('2014-04-21', tz='UTC'),  # Easter Monday
            pd.Timestamp('2014-04-25', tz='UTC'),  # Anzac Day
            pd.Timestamp('2014-06-02', tz='UTC'),  # Queen's Birthday
            pd.Timestamp('2014-10-27', tz='UTC'),  # Labour Day
            pd.Timestamp('2014-12-25', tz='UTC'),  # Christmas
            pd.Timestamp('2014-12-26', tz='UTC'),  # Boxing Day
        ]

        for session_label in expected_holidays_2014:
            self.assertNotIn(session_label, self.calendar.all_sessions)

        early_closes_2014 = [
            # Business day prior to Christmas
            pd.Timestamp('2014-12-24', tz='UTC'),
            # Business day prior to New Year's Day
            pd.Timestamp('2014-12-31', tz='UTC'),
        ]

        for early_close_session_label in early_closes_2014:
            self.assertIn(
                early_close_session_label,
                self.calendar.early_closes,
            )

    def test_christmas_and_boxing_day_special_cases(self):
        #    December 2010
        # Su Mo Tu We Th Fr Sa
        #           1  2  3  4
        #  5  6  7  8  9 10 11
        # 12 13 14 15 16 17 18
        # 19 20 21 22 23 24 25
        # 26 27 28 29 30 31
        #
        # Prior to 2011, the trading day before Christmas is a full
        # trading day.
        self.assertTrue(
            self.calendar.is_session(pd.Timestamp('2010-12-24', tz='UTC'))
        )
        self.assertEqual(
            self.calendar.session_close(
                pd.Timestamp('2010-12-24', tz='UTC')
            ),
            pd.Timestamp('2010-12-24 16:45', tz='NZ'),
        )

        # Saturday, but also Christmas Day.
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2010-12-25', tz='UTC'))
        )
        # Sunday, but also Boxing Day.
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2010-12-26', tz='UTC'))
        )
        # Since both Christmas and Boxing Day fell on the weekend, 2
        # additional holidays are observed.
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2010-12-27', tz='UTC'))
        )
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2010-12-28', tz='UTC'))
        )

        #    December 2011
        # Su Mo Tu We Th Fr Sa
        #              1  2  3
        #  4  5  6  7  8  9 10
        # 11 12 13 14 15 16 17
        # 18 19 20 21 22 23 24
        # 25 26 27 28 29 30 31
        #
        # Friday the 23rd is the trading day before Christmas, so it's a
        # half day (beginning in 2011.)
        self.assertTrue(
            self.calendar.is_session(pd.Timestamp('2011-12-23', tz='UTC'))
        )
        self.assertEqual(
            self.calendar.session_close(
                pd.Timestamp('2011-12-23', tz='UTC')
            ),
            pd.Timestamp('2011-12-23 12:45', tz='NZ'),
        )

        # Saturday.
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2011-12-24', tz='UTC'))
        )
        # Sunday, but also Christmas Day.
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2011-12-25', tz='UTC'))
        )
        # Boxing Day.
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2011-12-26', tz='UTC'))
        )
        # Since Christmas was on a Sunday, an additional holiday is
        # observed the day after Boxing Day.
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2011-12-27', tz='UTC'))
        )

    def test_new_years_day_and_day_after_special_cases(self):
        #    December 2010
        # Su Mo Tu We Th Fr Sa
        #           1  2  3  4
        #  5  6  7  8  9 10 11
        # 12 13 14 15 16 17 18
        # 19 20 21 22 23 24 25
        # 26 27 28 29 30 31
        #
        #     January 2011
        # Su Mo Tu We Th Fr Sa
        #                    1
        #  2  3  4  5  6  7  8
        #  9 10 11 12 13 14 15
        # 16 17 18 19 20 21 22
        # 23 24 25 26 27 28 29
        # 30 31
        #
        # Prior to 2011, the trading day before New Year's Day is a full
        # trading day.
        self.assertTrue(
            self.calendar.is_session(pd.Timestamp('2010-12-31', tz='UTC'))
        )
        self.assertEqual(
            self.calendar.session_close(
                pd.Timestamp('2010-12-31', tz='UTC')
            ),
            pd.Timestamp('2010-12-31 16:45', tz='NZ'),
        )

        # Saturday, but also New Year's Day.
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2011-01-01', tz='UTC'))
        )
        # Sunday, but also the day after New Year's Day.
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2011-01-02', tz='UTC'))
        )
        # Since both New Year's and the day after fell on the weekend, 2
        # additional holidays are observed.
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2011-01-03', tz='UTC'))
        )
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2011-01-04', tz='UTC'))
        )

        #    December 2011
        # Su Mo Tu We Th Fr Sa
        #              1  2  3
        #  4  5  6  7  8  9 10
        # 11 12 13 14 15 16 17
        # 18 19 20 21 22 23 24
        # 25 26 27 28 29 30 31
        #
        #     January 2012
        # Su Mo Tu We Th Fr Sa
        #  1  2  3  4  5  6  7
        #  8  9 10 11 12 13 14
        # 15 16 17 18 19 20 21
        # 22 23 24 25 26 27 28
        # 29 30 31
        #
        # Friday the 30th is the trading day before New Year's Day, so it's a
        # half day (beginning in 2011.)
        self.assertTrue(
            self.calendar.is_session(pd.Timestamp('2011-12-30', tz='UTC'))
        )
        self.assertEqual(
            self.calendar.session_close(
                pd.Timestamp('2011-12-30', tz='UTC')
            ),
            pd.Timestamp('2011-12-30 12:45', tz='NZ'),
        )

        # Saturday.
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2011-12-31', tz='UTC'))
        )
        # Sunday, but also New Year's Day.
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2012-01-01', tz='UTC'))
        )
        # The day after New Year's Day.
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2012-01-02', tz='UTC'))
        )
        # Since New Year's Day was on a Sunday, an additional holiday is
        # observed two days later.
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2012-01-03', tz='UTC'))
        )

    def test_weekend_to_monday_holidays(self):
        # Prior to 2015, Waitangi Day and Anzac Day are not "Mondayized",
        # that is, if they occur on the weekend, there is no make-up.

        #    February 2010
        # Su Mo Tu We Th Fr Sa
        #     1  2  3  4  5  6
        #  7  8  9 10 11 12 13
        # 14 15 16 17 18 19 20
        # 21 22 23 24 25 26 27
        # 28
        #
        # 2010-02-06 falls on a Saturday, so there is no missed session
        # for Waitangi Day.
        self.assertTrue(
            self.calendar.is_session(pd.Timestamp('2010-02-08', tz='UTC'))
        )

        #      April 2010
        # Su Mo Tu We Th Fr Sa
        #              1  2  3
        #  4  5  6  7  8  9 10
        # 11 12 13 14 15 16 17
        # 18 19 20 21 22 23 24
        # 25 26 27 28 29 30
        #
        # 2010-04-25 falls on a Sunday, so there is no missed session
        # for Anzac Day.
        self.assertTrue(
            self.calendar.is_session(pd.Timestamp('2010-04-26', tz='UTC'))
        )

        # Starting in 2015 Waitangi Day and Anzac Day are "Mondayized",
        # that is, if they fall on the weekend, they are observed on the
        # following Monday.

        #    February 2016
        # Su Mo Tu We Th Fr Sa
        #     1  2  3  4  5  6
        #  7  8  9 10 11 12 13
        # 14 15 16 17 18 19 20
        # 21 22 23 24 25 26 27
        # 28 29
        #
        # 2016-02-06 falls on a Saturday, so Waitangi Day is observed the
        # following Monday.
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2016-02-08', tz='UTC'))
        )

        #      April 2015
        # Su Mo Tu We Th Fr Sa
        #           1  2  3  4
        #  5  6  7  8  9 10 11
        # 12 13 14 15 16 17 18
        # 19 20 21 22 23 24 25
        # 26 27 28 29 30
        #
        # 2015-04-25 falls on a Saturday, so Anzac Day is observed the
        # following Monday.
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2015-04-27', tz='UTC'))
        )



#
# Copyright 2015 Quantopian, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from trading_calendars.utils.memoize import lazyval


class ZiplineCalendarError(Exception):
    msg = None

    def __init__(self, **kwargs):
        self.kwargs = kwargs

    @lazyval
    def message(self):
        return str(self)

    def __str__(self):
        msg = self.msg.format(**self.kwargs)
        return msg

    __unicode__ = __str__
    __repr__ = __str__


class InvalidCalendarName(ZiplineCalendarError):
    """
    Raised when a calendar with an invalid name is requested.
    """
    msg = (
        "The requested TradingCalendar, {calendar_name}, does not exist."
    )


class CalendarNameCollision(ZiplineCalendarError):
    """
    Raised when the static calendar registry already has a calendar with a
    given name.
    """
    msg = (
        "A calendar with the name {calendar_name} is already registered."
    )


class CyclicCalendarAlias(ZiplineCalendarError):
    """
    Raised when calendar aliases form a cycle.
    """
    msg = "Cycle in calendar aliases: [{cycle}]"


class ScheduleFunctionWithoutCalendar(ZiplineCalendarError):
    """
    Raised when schedule_function is called but there is not a calendar to be
    used in the construction of an event rule.
    """
    # TODO update message when new TradingSchedules are built
    msg = (
        "To use schedule_function, the TradingAlgorithm must be running on an "
        "ExchangeTradingSchedule, rather than {schedule}."
    )


class ScheduleFunctionInvalidCalendar(ZiplineCalendarError):
    """
    Raised when schedule_function is called with an invalid calendar argument.
    """
    msg = (
        "Invalid calendar '{given_calendar}' passed to schedule_function. "
        "Allowed options are {allowed_calendars}."
    )



#
# Copyright 2018 Quantopian, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import time

from pandas.tseries.holiday import (
    Holiday,
    GoodFriday,
    EasterMonday,
    weekend_to_monday,
)
from pytz import timezone

from .common_holidays import (
    new_years_day,
    epiphany,
    european_labour_day,
    assumption_day,
    all_saints_day,
    immaculate_conception,
    christmas_eve,
    christmas,
    boxing_day,
    new_years_eve,
)
from .trading_calendar import (
    HolidayCalendar,
    TradingCalendar,
    WEEKDAYS,
)

NewYearsDay = new_years_day()

Epiphany = epiphany(end_date='2007')

LabourDay = european_labour_day()

AssumptionDay = assumption_day(end_date='2005', observance=weekend_to_monday)

NationalDay = Holiday(
    'National Day',
    month=10,
    day=12,
    end_date='2005',
)

AllSaintsDay = all_saints_day(end_date='2005')

ConstitutionDay = Holiday(
    'Constitution Day',
    month=12,
    day=6,
    end_date='2005',
)

ImmaculateConception = immaculate_conception(end_date='2005')

ChristmasEveThrough2010 = christmas_eve(end_date='2011')
ChristmasEveEarlyClose2012Onwards = christmas_eve(
    start_date='2012',
    days_of_week=(WEEKDAYS),
)
Christmas = christmas()
BoxingDay = boxing_day()

NewYearsEveThrough2010 = new_years_eve(end_date='2011')
NewYearsEveEarlyClose2012Onwards = new_years_eve(
    start_date='2012',
    days_of_week=(WEEKDAYS),
)


class XMADExchangeCalendar(TradingCalendar):
    """
    Calendar for the Madrid Stock Exchange (Bolsa de Madrid).

    Open Time: 9:00 AM, CET (Central European Time)
    Close Time: 5:30 PM, CET (Central European Time)

    Regularly-Observed Holidays:
      - New Year's Day
      - Good Friday
      - Easter Monday
      - Labour Day
      - Christmas Day
      - Boxing Day

    Holidays No Longer Observed:
      - Epiphany (until 2006, inclusive)
      - Assumption Day (until 2004, inclusive)
      - National Day (until 2004, inclusive)
      - All Saints Day (until 2004, inclusive)
      - Constitution Day (until 2004, inclusive)
      - Immaculate Conception (until 2004, inclusive)
      - Christmas Eve (until 2010, inclusive)
      - New Year's Eve (until 2010, inclusive)

    Early Closes:
      - Christmas Eve (2012 and after)
      - New Year's Eve (2012 and after)
    """
    regular_early_close = time(14, 00)

    name = 'XMAD'

    tz = timezone('Europe/Madrid')

    open_times = (
        (None, time(9, 1)),
    )

    close_times = (
        (None, time(17, 30)),
    )

    @property
    def regular_holidays(self):
        return HolidayCalendar([
            NewYearsDay,
            Epiphany,
            GoodFriday,
            EasterMonday,
            LabourDay,
            AssumptionDay,
            NationalDay,
            AllSaintsDay,
            ConstitutionDay,
            ImmaculateConception,
            ChristmasEveThrough2010,
            Christmas,
            BoxingDay,
            NewYearsEveThrough2010,
        ])

    @property
    def special_closes(self):
        return [
            (
                self.regular_early_close,
                HolidayCalendar([
                    ChristmasEveEarlyClose2012Onwards,
                    NewYearsEveEarlyClose2012Onwards,
                ])
            )
        ]



import numpy as np

NANOSECONDS_PER_MINUTE = int(6e10)


def next_divider_idx(dividers, minute_val):

    divider_idx = np.searchsorted(dividers, minute_val, side="right")
    target = dividers[divider_idx]

    if minute_val == target:
        # if dt is exactly on the divider, go to the next value
        return divider_idx + 1
    else:
        return divider_idx


def previous_divider_idx(dividers, minute_val):

    divider_idx = np.searchsorted(dividers, minute_val)

    if divider_idx == 0:
        raise ValueError("Cannot go earlier in calendar!")

    return divider_idx - 1


def is_open(opens, closes, minute_val):

    open_idx = np.searchsorted(opens, minute_val)
    close_idx = np.searchsorted(closes, minute_val)

    if open_idx != close_idx:
        # if the indices are not same, that means the market is open
        return True
    else:
        try:
            # if they are the same, it might be the first minute of a
            # session
            return minute_val == opens[open_idx]
        except IndexError:
            # this can happen if we're outside the schedule's range (like
            # after the last close)
            return False


def compute_all_minutes(opens_in_ns, closes_in_ns):
    """
    Given arrays of opens and closes, both in nanoseconds,
    return an array of each minute between the opens and closes.
    """
    deltas = closes_in_ns - opens_in_ns

    # + 1 because we want 390 mins per standard day, not 389
    daily_sizes = (deltas // NANOSECONDS_PER_MINUTE) + 1
    num_minutes = daily_sizes.sum()

    # One allocation for the entire thing. This assumes that each day
    # represents a contiguous block of minutes.
    pieces = []

    for open_, size in zip(opens_in_ns, daily_sizes):
        pieces.append(
            np.arange(open_,
                      open_ + size * NANOSECONDS_PER_MINUTE,
                      NANOSECONDS_PER_MINUTE)
        )

    out = np.concatenate(pieces).view('datetime64[ns]')
    assert len(out) == num_minutes
    return out



from unittest import TestCase
import pandas as pd

from .test_trading_calendar import ExchangeCalendarTestBase
from trading_calendars.exchange_calendar_cmes import CMESExchangeCalendar


class CMESCalendarTestCase(ExchangeCalendarTestBase, TestCase):
    answer_key_filename = "cmes"
    calendar_class = CMESExchangeCalendar
    GAPS_BETWEEN_SESSIONS = False
    MAX_SESSION_HOURS = 24

    def test_2016_holidays(self):
        # good friday: 2016-03-25
        # christmas (observed)_: 2016-12-26
        # new years (observed): 2016-01-02
        for date in ["2016-03-25", "2016-12-26", "2016-01-02"]:
            self.assertFalse(
                self.calendar.is_session(pd.Timestamp(date, tz='UTC'))
            )

    def test_2016_early_closes(self):
        # mlk day: 2016-01-18
        # presidents: 2016-02-15
        # mem day: 2016-05-30
        # july 4: 2016-07-04
        # labor day: 2016-09-05
        # thankgiving: 2016-11-24
        for date in ["2016-01-18", "2016-02-15", "2016-05-30", "2016-07-04",
                     "2016-09-05", "2016-11-24"]:
            dt = pd.Timestamp(date, tz='UTC')
            self.assertTrue(dt in self.calendar.early_closes)

            market_close = self.calendar.schedule.loc[dt].market_close
            self.assertEqual(
                12,
                market_close.tz_localize('UTC').tz_convert(
                    self.calendar.tz
                ).hour
            )



from unittest import TestCase
import pandas as pd

from .test_trading_calendar import ExchangeCalendarTestBase
from trading_calendars.exchange_calendar_xcbf import XCBFExchangeCalendar


class XCBFCalendarTestCase(ExchangeCalendarTestBase, TestCase):
    answer_key_filename = "xcbf"
    calendar_class = XCBFExchangeCalendar

    MAX_SESSION_HOURS = 8

    def test_2016_holidays(self):
        # new years: jan 1
        # mlk: jan 18
        # presidents: feb 15
        # good friday: mar 25
        # mem day: may 30
        # independence day: july 4
        # labor day: sep 5
        # thanksgiving day: nov 24
        # christmas (observed): dec 26
        # new years (observed): jan 2 2017
        for day in ["2016-01-01", "2016-01-18", "2016-02-15", "2016-03-25",
                    "2016-05-30", "2016-07-04", "2016-09-05", "2016-11-24",
                    "2016-12-26", "2017-01-02"]:
            self.assertFalse(
                self.calendar.is_session(pd.Timestamp(day, tz='UTC'))
            )

    def test_2016_early_closes(self):
        # only early close is day after thanksgiving: nov 25
        dt = pd.Timestamp("2016-11-25", tz='UTC')
        self.assertTrue(dt in self.calendar.early_closes)

        market_close = self.calendar.schedule.loc[dt].market_close
        market_close = market_close.tz_localize("UTC").tz_convert(
            self.calendar.tz
        )
        self.assertEqual(12, market_close.hour)
        self.assertEqual(15, market_close.minute)

    def test_adhoc_holidays(self):
        # hurricane sandy: oct 29 2012, oct 30 2012
        # national days of mourning:
        # - apr 27 1994
        # - june 11 2004
        # - jan 2 2007
        for day in ["1994-04-27", "2004-06-11", "2007-01-02",
                    "2012-10-29", "2012-10-30"]:
            self.assertFalse(
                self.calendar.is_session(pd.Timestamp(day, tz='UTC'))
            )



"""
Note
----
Lifted from zipline.
"""
from weakref import WeakKeyDictionary


class lazyval(object):
    """Decorator that marks that an attribute of an instance should not be
    computed until needed, and that the value should be memoized.

    Example
    -------

    >>> from zipline.utils.memoize import lazyval
    >>> class C(object):
    ...     def __init__(self):
    ...         self.count = 0
    ...     @lazyval
    ...     def val(self):
    ...         self.count += 1
    ...         return "val"
    ...
    >>> c = C()
    >>> c.count
    0
    >>> c.val, c.count
    ('val', 1)
    >>> c.val, c.count
    ('val', 1)
    >>> c.val = 'not_val'
    Traceback (most recent call last):
    ...
    AttributeError: Can't set read-only attribute.
    >>> c.val
    'val'
    """
    def __init__(self, get):
        self._get = get
        self._cache = WeakKeyDictionary()

    def __get__(self, instance, owner):
        if instance is None:
            return self
        try:
            return self._cache[instance]
        except KeyError:
            self._cache[instance] = val = self._get(instance)
            return val

    def __set__(self, instance, value):
        raise AttributeError("Can't set read-only attribute.")

    def __delitem__(self, instance):
        del self._cache[instance]



from unittest import TestCase
import pandas as pd

from .test_trading_calendar import ExchangeCalendarTestBase
from trading_calendars.exchange_calendar_xhel import XHELExchangeCalendar


class XHELCalendarTestCase(ExchangeCalendarTestBase, TestCase):

    answer_key_filename = 'xhel'
    calendar_class = XHELExchangeCalendar

    # The XHEL is open from 10:00 am to 6:30 pm.
    MAX_SESSION_HOURS = 8.5

    HAVE_EARLY_CLOSES = False

    def test_all_holidays(self):
        all_sessions = self.calendar.all_sessions

        expected_holidays = [
            pd.Timestamp('2018-01-01', tz='UTC'),  # New Year's Day
            pd.Timestamp('2017-01-06', tz='UTC'),  # Epiphany
            pd.Timestamp('2018-03-30', tz='UTC'),  # Good Friday
            pd.Timestamp('2018-04-02', tz='UTC'),  # Easter Monday
            pd.Timestamp('2018-05-01', tz='UTC'),  # Labour Day
            pd.Timestamp('2018-05-10', tz='UTC'),  # Ascension Day
            pd.Timestamp('2018-06-22', tz='UTC'),  # Midsummer Eve
            pd.Timestamp('2018-12-06', tz='UTC'),  # Finland Independence Day
            pd.Timestamp('2018-12-24', tz='UTC'),  # Christmas Eve
            pd.Timestamp('2018-12-25', tz='UTC'),  # Christmas Day
            pd.Timestamp('2018-12-26', tz='UTC'),  # Boxing Day
            pd.Timestamp('2018-12-31', tz='UTC'),  # New Year's Eve
        ]

        for session_label in expected_holidays:
            self.assertNotIn(session_label, all_sessions)

        # The market holiday for Midsummer Eve should fall on the Friday after
        # June 18. In 2010, June 18 was a Friday so the market holiday should
        # be on June 25.
        self.assertIn(pd.Timestamp('2010-06-18', tz='UTC'), all_sessions)
        self.assertNotIn(pd.Timestamp('2010-06-25', tz='UTC'), all_sessions)

    def test_holidays_fall_on_weekend(self):
        all_sessions = self.calendar.all_sessions

        # Holidays falling on a weekend should not be made up during the week.
        expected_sessions = [
            # In 2018, the Epiphany fell on a Saturday, so the market should be
            # open on both the prior Friday and the following Monday.
            pd.Timestamp('2018-01-05', tz='UTC'),
            pd.Timestamp('2018-01-08', tz='UTC'),

            # In 2010, Labour Day fell on a Saturday, so the market should be
            # open on both the prior Friday and the following Monday.
            pd.Timestamp('2010-04-30', tz='UTC'),
            pd.Timestamp('2010-05-03', tz='UTC'),

            # In 2015, Finland Independence Day fell on a Sunday, so the market
            # should be open on both the prior Friday and the following Monday.
            pd.Timestamp('2015-12-04', tz='UTC'),
            pd.Timestamp('2015-12-07', tz='UTC'),

            # In 2010, Christmas fell on a Saturday, meaning Boxing Day fell on
            # a Sunday. The market should thus be open on the following Monday.
            pd.Timestamp('2010-12-27', tz='UTC'),

            # In 2017, New Year's Day fell on a Sunday, so the market should be
            # open on both the prior Friday and the following Monday.
            pd.Timestamp('2016-12-30', tz='UTC'),
            pd.Timestamp('2017-01-02', tz='UTC'),
        ]

        for session_label in expected_sessions:
            self.assertIn(session_label, self.calendar.all_sessions)

        # In 2017, June 18 fell on a Sunday, so the following Friday should be
        # a holiday.
        self.assertNotIn(pd.Timestamp('2017-06-23', tz='UTC'), all_sessions)



#
# Copyright 2018 Quantopian, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import time

from pandas.tseries.holiday import Holiday, GoodFriday, EasterMonday
from pytz import timezone

from .common_holidays import (
    new_years_day,
    holy_wednesday,
    maundy_thursday,
    ascension_day,
    whit_monday,
    european_labour_day,
    christmas_eve,
    christmas,
    boxing_day,
    new_years_eve,
)
from .trading_calendar import HolidayCalendar, TradingCalendar, WEEKDAYS

NewYearsDay = new_years_day()

HolyWednesday = holy_wednesday(start_date='2011', days_of_week=WEEKDAYS)
MaundyThursday = maundy_thursday()
AscensionDay = ascension_day()
WhitMonday = whit_monday()

LabourDay = european_labour_day()

ConstitutionDay = Holiday('Constitution Day', month=5, day=17)

ChristmasEve = christmas_eve()
Christmas = christmas()
BoxingDay = boxing_day()

NewYearsEve = new_years_eve()


class XOSLExchangeCalendar(TradingCalendar):
    """
    Calendar for the Oslo Stock Exchange in Norway.

    Open Time: 9:00 AM, CET (Central European Time)
    Close Time: 4:20 PM, CET (Central European Time)

    Regularly-Observed Holidays:
      - New Year's Day
      - Maundy Thursday
      - Good Friday
      - Easter Monday
      - Labour Day
      - Ascension Day
      - Constitution Day
      - Whit Monday
      - Christmas Eve
      - Christmas Day
      - Boxing Day
      - New Year's Eve

    Early Closes:
      - Holy Wednesday
    """
    name = 'XOSL'
    tz = timezone('Europe/Oslo')
    open_times = (
        (None, time(9, 1)),
    )
    close_times = (
        (None, time(16, 20)),
    )
    regular_early_close = time(13)

    @property
    def regular_holidays(self):
        return HolidayCalendar([
            NewYearsDay,
            MaundyThursday,
            GoodFriday,
            EasterMonday,
            LabourDay,
            AscensionDay,
            ConstitutionDay,
            WhitMonday,
            ChristmasEve,
            Christmas,
            BoxingDay,
            NewYearsEve,
        ])

    @property
    def special_closes(self):
        return [(self.regular_early_close, HolidayCalendar([HolyWednesday]))]



#
# Copyright 2018 Quantopian, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import time
from pandas import Timestamp
from pandas.tseries.holiday import (
    EasterMonday,
    GoodFriday,
    Holiday,
    previous_workday
)
from pytz import timezone

from .common_holidays import (
    new_years_day,
    european_labour_day,
    whit_monday,
    christmas_eve,
    christmas,
    boxing_day,
    new_years_eve,

)
from .trading_calendar import (
    TradingCalendar,
    HolidayCalendar
)

# Regular Holidays
# ----------------
NewYearsDay = new_years_day()

EuropeanLabourDay = european_labour_day()

# Whit Monday observed in 2007, before it became regularly observed
# starting in 2015.
WhitMonday2007AdHoc = Timestamp('2007-05-28', tz='UTC')

WhitMonday = whit_monday(start_date='2015-01-01')

DayOfGermanUnity = Holiday(
    "Day of German Unity",
    month=10,
    day=3,
    start_date='2014-01-01'
)

# Reformation Day was a German national holiday in 2017.
ReformationDay500thAnniversaryAdHoc = Timestamp('2017-10-31', tz='UTC')

ChristmasEve = christmas_eve()

Christmas = christmas()

BoxingDay = boxing_day()

NewYearsEve = new_years_eve()

# Early Closes
# ------------
# The last weekday before Dec 31 is an early close starting in 2010.
LastWorkingDay = Holiday(
    "Last Working Day of Year Early Close",
    month=12,
    day=31,
    start_date='2010-01-01',
    observance=previous_workday,
)


class XFRAExchangeCalendar(TradingCalendar):
    """
    Exchange calendar for the Frankfurt Stock Exchange (XFRA).

    Open Time: 9:00 AM, CET
    Close Time: 5:30 PM, CET

    Regularly-Observed Holidays:
    - New Years Day
    - Good Friday
    - Easter Monday
    - Whit Monday
    - Labour Day
    - Day of German Unity
    - Christmas Eve
    - Christmas Day
    - Boxing Day

    Early Closes:
    - Last working day before Dec. 31
    """
    # TODO: verify the early close time
    regular_early_close = time(12, 30)

    name = 'XFRA'

    tz = timezone('CET')

    open_times = (
        (None, time(9, 1)),
    )

    close_times = (
        (None, time(17, 30)),
    )

    @property
    def regular_holidays(self):
        return HolidayCalendar([
            NewYearsDay,
            GoodFriday,
            EasterMonday,
            EuropeanLabourDay,
            WhitMonday,
            DayOfGermanUnity,
            ChristmasEve,
            Christmas,
            BoxingDay,
            NewYearsEve,
        ])

    @property
    def adhoc_holidays(self):
        return [
            WhitMonday2007AdHoc,
            ReformationDay500thAnniversaryAdHoc,
        ]

    @property
    def special_closes(self):
        return [
            (self.regular_early_close, HolidayCalendar([
                LastWorkingDay,
            ]))
        ]



from unittest import TestCase

import pandas as pd
from pandas.util.testing import assert_index_equal

from trading_calendars.weekday_calendar import WeekdayCalendar
from .test_trading_calendar import ExchangeCalendarTestBase


class WeekdayCalendarTestCase(ExchangeCalendarTestBase, TestCase):

    answer_key_filename = '24-5'
    calendar_class = WeekdayCalendar
    start_date = pd.Timestamp('2018-01-01', tz='UTC')
    end_date = pd.Timestamp('2018-12-31', tz='UTC')
    MAX_SESSION_HOURS = 24
    GAPS_BETWEEN_SESSIONS = False
    HAVE_EARLY_CLOSES = False

    MINUTE_INDEX_TO_SESSION_LABELS_START = pd.Timestamp('2018-01-01', tz='UTC')
    MINUTE_INDEX_TO_SESSION_LABELS_END = pd.Timestamp('2018-04-04', tz='UTC')

    DAYLIGHT_SAVINGS_DATES = ['2018-04-05', '2018-11-01']

    def get_session_block(self):
        # This range is chosen specifically because it is "enclosed" by
        # adjacent days that are also sessions. This prevents any edge case
        # issues when looking at market opens or closes.
        return self.calendar.all_sessions[1:4]

    def test_open_every_weekday(self):
        calendar = self.calendar

        dates = pd.date_range(self.start_date, self.end_date, tz='UTC')
        assert_index_equal(
            calendar.sessions_in_range(dates[0], dates[-1]),
            # The pandas weekday is defined as Monday=0 to Sunday=6.
            dates[dates.weekday <= 4],
        )

    def test_open_every_weekday_minute(self):
        calendar = self.calendar

        minutes = pd.date_range(
            self.start_date,
            # Our calendar should include all the minutes of this last session.
            self.end_date + pd.Timedelta('1 Day') - pd.Timedelta('1 Minute'),
            freq='min',
            tz='UTC',
        )
        assert_index_equal(
            calendar.minutes_for_sessions_in_range(
                self.start_date,
                self.end_date,
            ),
            # The pandas weekday is defined as Monday=0 to Sunday=6.
            minutes[minutes.weekday <= 4],
        )



#
# Copyright 2018 Quantopian, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import time

from pandas.tseries.holiday import Holiday, GoodFriday, EasterMonday
from pytz import timezone

from .common_holidays import (
    new_years_day,
    epiphany,
    european_labour_day,
    ascension_day,
    midsummer_eve,
    christmas_eve,
    christmas,
    boxing_day,
    new_years_eve,
)
from .trading_calendar import HolidayCalendar, TradingCalendar

NewYearsDay = new_years_day()

Epiphany = epiphany()

LabourDay = european_labour_day()

AscensionDay = ascension_day()

MidsummerEve = midsummer_eve()

IndependenceDay = Holiday('Finland Independence Day', month=12, day=6)

ChristmasEve = christmas_eve()
Christmas = christmas()
BoxingDay = boxing_day()

NewYearsEve = new_years_eve()


class XHELExchangeCalendar(TradingCalendar):
    """
    Calendar for the Helsinki Stock Exchange in Finland.

    Open Time: 10:00 AM, CET (Eastern European Time)
    Close Time: 6:30 PM, CET (Eastern European Time)

    Regularly-Observed Holidays:
      - New Year's Day
      - Epiphany
      - Good Friday
      - Easter Monday
      - Labour Day
      - Ascension Day
      - Midsummer Eve
      - Independence Day
      - Christmas Eve
      - Christmas Day
      - Boxing Day
      - New Year's Eve

    Early Closes:
      - None
    """
    name = 'XHEL'
    tz = timezone('Europe/Helsinki')
    open_times = (
        (None, time(10, 1)),
    )
    close_times = (
        (None, time(18, 30)),
    )

    @property
    def regular_holidays(self):
        return HolidayCalendar([
            NewYearsDay,
            Epiphany,
            GoodFriday,
            EasterMonday,
            LabourDay,
            AscensionDay,
            MidsummerEve,
            IndependenceDay,
            ChristmasEve,
            Christmas,
            BoxingDay,
            NewYearsEve,
        ])



from unittest import TestCase
import pandas as pd

from .test_trading_calendar import ExchangeCalendarTestBase
from trading_calendars.exchange_calendar_bvmf import BVMFExchangeCalendar


class BVMFCalendarTestCase(ExchangeCalendarTestBase, TestCase):

    answer_key_filename = 'bvmf'
    calendar_class = BVMFExchangeCalendar

    MINUTE_INDEX_TO_SESSION_LABELS_START = pd.Timestamp('2011-01-05', tz='UTC')
    MINUTE_INDEX_TO_SESSION_LABELS_END = pd.Timestamp('2011-04-05', tz='UTC')

    HAVE_EARLY_CLOSES = False
    HAVE_LATE_OPENS = True

    DAYLIGHT_SAVINGS_DATES = ['2017-02-17', '2017-10-16']

    # Open from 10am to 5pm
    MAX_SESSION_HOURS = 7

    def test_normal_year(self):
        expected_holidays_2017 = [
            pd.Timestamp('2017-01-25', tz='UTC'),  # Sao Paolo City Anniversary
            pd.Timestamp('2017-02-27', tz='UTC'),  # Carnival
            pd.Timestamp('2017-02-28', tz='UTC'),  # Carnival
            pd.Timestamp('2017-04-14', tz='UTC'),  # Good Friday
            pd.Timestamp('2017-04-21', tz='UTC'),  # Tiradentes Day
            pd.Timestamp('2017-05-01', tz='UTC'),  # Labor Day
            pd.Timestamp('2017-06-15', tz='UTC'),  # Corpus Christi Day
            pd.Timestamp('2017-09-07', tz='UTC'),  # Independence Day
            pd.Timestamp('2017-10-12', tz='UTC'),  # Our Lady of Aparecida Day
            pd.Timestamp('2017-11-02', tz='UTC'),  # All Souls Day
            pd.Timestamp('2017-11-15', tz='UTC'),  # Proclamation of the
                                                   #  Republic Day
            pd.Timestamp('2017-11-20', tz='UTC'),  # Black Consciousness Day
            pd.Timestamp('2017-12-25', tz='UTC'),  # Christmas Day
            pd.Timestamp('2017-12-29', tz='UTC'),  # Day before New Years
        ]

        for session_label in expected_holidays_2017:
            self.assertNotIn(session_label, self.calendar.all_sessions)

        other_holidays = [
            pd.Timestamp("2018-01-01", tz='UTC'),  # New Year's Day
            pd.Timestamp("2015-07-09", tz='UTC'),  # Constitutional
                                                   #  Revolution Day
        ]

        for session_label in other_holidays:
            self.assertNotIn(session_label, self.calendar.all_sessions)

    def test_late_opens(self):
        # Ash Wednesday, 46 days before Easter Sunday
        late_opens = [
            pd.Timestamp("2016-02-10", tz='UTC'),
            pd.Timestamp("2017-03-01", tz='UTC'),
            pd.Timestamp("2018-02-14", tz='UTC'),
        ]

        for late_open_session_label in late_opens:
            self.assertIn(
                late_open_session_label,
                self.calendar.late_opens,
            )

    def test_special_holidays(self):
        # Constitutionalist Revolution started in 1998
        self.assertNotIn(
            pd.Timestamp("1998-07-09", tz='UTC'),
            self.calendar.all_sessions
        )

        self.assertIn(
            pd.Timestamp("1997-07-09", tz='UTC'),
            self.calendar.all_sessions
        )

        # Day of Black Awareness started in 2004
        self.assertNotIn(
            pd.Timestamp("2006-11-20", tz='UTC'),
            self.calendar.all_sessions
        )

        self.assertIn(
            pd.Timestamp("2003-11-20", tz='UTC'),
            self.calendar.all_sessions
        )

    def test_day_before_new_year(self):
        # if Jan 1 is Tuesday through Saturday, we are closed the day before.
        # if Jan 1 is Monday or Sunday, we are closed the Friday before.

        # 2018: Jan 1 is Monday, so Friday 12/29 should be closed
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2017-12-29', tz='UTC'))
        )

        # 2017: Jan 1 is Sunday, so Friday 12/30 should be closed
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2016-12-30', tz='UTC'))
        )

        # 2011: Jan 1 is Saturday, so Friday 12/31 should be closed
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2010-12-31', tz='UTC'))
        )

        # 2014: Jan 1 is Wednesday, so Tuesday 12/31 should be closed
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2013-12-31', tz='UTC'))
        )

    def test_world_cup(self):
        self.assertFalse(
            self.calendar.is_session(pd.Timestamp('2014-06-12', tz='UTC'))
        )



from unittest import TestCase
import pandas as pd

from .test_trading_calendar import ExchangeCalendarTestBase
from trading_calendars.exchange_calendar_xfra import XFRAExchangeCalendar


class XFRACalendarTestCase(ExchangeCalendarTestBase, TestCase):

    answer_key_filename = 'xfra'
    calendar_class = XFRAExchangeCalendar

    # The FWB is open from 9:00 am to 5:30 pm.
    MAX_SESSION_HOURS = 8.5

    def test_whit_monday(self):
        # Whit Monday was not observed prior to 2007.
        self.assertIn(
            pd.Timestamp('2006-06-05', tz='UTC'),
            self.calendar.all_sessions,
        )

        # It was observed as a one-off in 2007...
        self.assertNotIn(
            pd.Timestamp('2007-05-28', tz='UTC'),
            self.calendar.all_sessions,
        )

        # ...then not again...
        self.assertIn(
            pd.Timestamp('2008-05-12', tz='UTC'),
            self.calendar.all_sessions,
        )

        # ...until 2015...
        self.assertNotIn(
            pd.Timestamp('2015-05-25', tz='UTC'),
            self.calendar.all_sessions,
        )

        # ...when it became regularly observed.
        self.assertNotIn(
            pd.Timestamp('2016-05-16', tz='UTC'),
            self.calendar.all_sessions,
        )

    def test_2012(self):
        expected_holidays_2012 = [
            # New Year's Day fell on a Sunday, so it is not a holiday this year
            pd.Timestamp("2012-04-06", tz='UTC'),  # Good Friday
            pd.Timestamp("2012-04-09", tz='UTC'),  # Easter Monday
            pd.Timestamp("2012-05-01", tz='UTC'),  # Labour Day
            # Whit Monday was observed in 2007, then 2015 and after.
            # German Unity Day started being celebrated in 2014
            pd.Timestamp("2012-12-24", tz='UTC'),  # Christmas Eve
            pd.Timestamp("2012-12-25", tz='UTC'),  # Christmas
            pd.Timestamp("2012-12-26", tz='UTC'),  # Boxing Day
            pd.Timestamp("2012-12-31", tz='UTC'),  # New Year's Eve
        ]

        for session_label in expected_holidays_2012:
            self.assertNotIn(session_label, self.calendar.all_sessions)

        early_closes_2012 = [
            pd.Timestamp("2012-12-28", tz='UTC'),  # Last working day of 2012
        ]

        for early_close_session_label in early_closes_2012:
            self.assertIn(early_close_session_label,
                          self.calendar.early_closes)

    def test_half_days(self):
        half_days = [
            # In 2011, NYE was on a Sat, so Fri is a half day
            pd.Timestamp('2011-12-30', tz='CET'),
            # In 2012, NYE was on a Mon, so the preceding Fri is a half day
            pd.Timestamp('2012-12-28', tz='CET'),
        ]

        for half_day in half_days:
            half_day_close_time = self.calendar.next_close(half_day)
            self.assertEqual(
                half_day_close_time,
                half_day + pd.Timedelta(hours=12, minutes=30)
            )

    def test_reformation_day(self):
        # Reformation Day was a German national holiday in 2017 only.
        self.assertNotIn(
            pd.Timestamp('2017-10-31', tz='UTC'),
            self.calendar.all_sessions,
        )

        # Ensure it is a trading day in the surrounding years.
        self.assertIn(
            pd.Timestamp('2016-10-31', tz='UTC'),
            self.calendar.all_sessions,
        )
        self.assertIn(
            pd.Timestamp('2018-10-31', tz='UTC'),
            self.calendar.all_sessions,
        )



#
# Copyright 2018 Quantopian, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import time

from pandas.tseries.holiday import (
    Holiday,
    GoodFriday,
    EasterMonday,
    previous_friday,
)
from pytz import timezone

from .common_holidays import (
    new_years_day,
    epiphany,
    ascension_day,
    whit_monday,
    corpus_christi,
    european_labour_day,
    assumption_day,
    all_saints_day,
    immaculate_conception,
    christmas_eve,
    christmas,
    new_years_eve,
)
from .trading_calendar import HolidayCalendar, TradingCalendar

NewYearsDay = new_years_day()

Epiphany = epiphany()

AscensionDay = ascension_day()
WhitMonday = whit_monday()
CorpusChristi = corpus_christi()

LabourDay = european_labour_day()

AssumptionDay = assumption_day()

NationalHoliday = Holiday('National Holiday', month=10, day=26)

AllSaintsDay = all_saints_day()

ImmaculateConception = immaculate_conception()

ChristmasEve = christmas_eve()
Christmas = christmas()

SaintStephensDay = Holiday("Saint Stephen's Day", month=12, day=26)

# Prior to 2016, when New Year's Eve fell on the weekend, it was observed
# on the preceding Friday. In 2016 and after, it is not made up.
NewYearsEveThrough2015 = new_years_eve(
    observance=previous_friday,
    end_date='2016',
)
NewYearsEve2016Onwards = new_years_eve(start_date='2016')


class XWBOExchangeCalendar(TradingCalendar):
    """
    Calendar for the Wiener Borse AG exchange in Vienna, Austria.

    Open Time: 9:00 AM, CET (Central European Time)
    Close Time: 5:30 PM, CET (Central European Time)

    Regularly-Observed Holidays:
      - New Year's Day
      - Epiphany
      - Good Friday
      - Easter Monday
      - Ascension Day
      - Whit Monday
      - Corpus Christi
      - Labour Day
      - Assumption Day
      - National Holiday
      - All Saints Day
      - Immaculate Conception
      - Christmas Eve
      - Christmas Day
      - St. Stephen's Day
      - New Year's Eve

    Early Closes:
      - None
    """
    name = 'XWBO'

    tz = timezone('Europe/Vienna')

    open_times = (
        (None, time(9, 1)),
    )

    close_times = (
        (None, time(17, 30)),
    )

    @property
    def regular_holidays(self):
        return HolidayCalendar([
            NewYearsDay,
            Epiphany,
            GoodFriday,
            EasterMonday,
            AscensionDay,
            WhitMonday,
            CorpusChristi,
            LabourDay,
            AssumptionDay,
            NationalHoliday,
            AllSaintsDay,
            ImmaculateConception,
            ChristmasEve,
            Christmas,
            SaintStephensDay,
            NewYearsEveThrough2015,
            NewYearsEve2016Onwards,
        ])



from unittest import TestCase
import pandas as pd

from .test_trading_calendar import ExchangeCalendarTestBase
from trading_calendars.exchange_calendar_xosl import XOSLExchangeCalendar


class XOSLCalendarTestCase(ExchangeCalendarTestBase, TestCase):

    answer_key_filename = 'xosl'
    calendar_class = XOSLExchangeCalendar

    # The XOSL is open from 9:00 am to 4:30 pm.
    MAX_SESSION_HOURS = 7.5

    def test_all_holidays(self):
        all_sessions = self.calendar.all_sessions

        expected_holidays = [
            pd.Timestamp('2018-01-01', tz='UTC'),  # New Year's Day
            pd.Timestamp('2018-03-29', tz='UTC'),  # Maundy Thursday
            pd.Timestamp('2018-03-30', tz='UTC'),  # Good Friday
            pd.Timestamp('2018-04-02', tz='UTC'),  # Easter Monday
            pd.Timestamp('2018-05-01', tz='UTC'),  # Labour Day
            pd.Timestamp('2018-05-10', tz='UTC'),  # Ascension Day
            pd.Timestamp('2018-05-17', tz='UTC'),  # Constitution Day
            pd.Timestamp('2018-05-21', tz='UTC'),  # Whit Monday
            pd.Timestamp('2018-12-24', tz='UTC'),  # Christmas Eve
            pd.Timestamp('2018-12-25', tz='UTC'),  # Christmas Day
            pd.Timestamp('2018-12-26', tz='UTC'),  # Boxing Day
            pd.Timestamp('2018-12-31', tz='UTC'),  # New Year's Eve
        ]

        for session_label in expected_holidays:
            self.assertNotIn(session_label, all_sessions)

    def test_holidays_fall_on_weekend(self):
        all_sessions = self.calendar.all_sessions

        # Holidays falling on a weekend should not be made up during the week.
        expected_sessions = [
            # In 2010, Labour Day fell on a Saturday, so the market should be
            # open on both the prior Friday and the following Monday.
            pd.Timestamp('2010-04-30', tz='UTC'),
            pd.Timestamp('2010-05-03', tz='UTC'),

            # In 2015, Constitution Day fell on a Sunday, so the market should
            # be open on both the prior Friday and the following Monday.
            pd.Timestamp('2015-05-15', tz='UTC'),
            pd.Timestamp('2015-05-18', tz='UTC'),

            # In 2010, Christmas fell on a Saturday, meaning Boxing Day fell on
            # a Sunday. The market should thus be open on the following Monday.
            pd.Timestamp('2010-12-27', tz='UTC'),

            # In 2017, New Year's Day fell on a Sunday, so the market should be
            # open on both the prior Friday and the following Monday.
            pd.Timestamp('2016-12-30', tz='UTC'),
            pd.Timestamp('2017-01-02', tz='UTC'),
        ]

        for session_label in expected_sessions:
            self.assertIn(session_label, all_sessions)

    def test_early_closes(self):
        # Starting in 2011, Holy Wednesday should be a half day.
        self.assertEqual(
            self.calendar.session_close(pd.Timestamp('2010-03-31', tz='UTC')),
            pd.Timestamp('2010-03-31 16:20', tz='Europe/Oslo'),
        )
        self.assertEqual(
            self.calendar.session_close(pd.Timestamp('2011-04-20', tz='UTC')),
            pd.Timestamp('2011-04-20 13:00', tz='Europe/Oslo'),
        )



#
# Copyright 2018 Quantopian, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import time

from dateutil.relativedelta import MO
from pandas import DateOffset, Timestamp
from pandas.tseries.holiday import (
    Holiday,
    GoodFriday,
    EasterMonday,
    previous_friday,
    sunday_to_monday,
    weekend_to_monday,
)
from pytz import timezone

from .common_holidays import (
    new_years_day,
    anzac_day,
    christmas,
    weekend_christmas,
    boxing_day,
    weekend_boxing_day,
)
from .trading_calendar import HolidayCalendar, TradingCalendar

NewYearsDay = new_years_day(observance=weekend_to_monday)

AustraliaDay = Holiday(
    'Australia Day',
    month=1,
    day=26,
    observance=weekend_to_monday,
)

# Anzac Day was observed on Monday when it fell on a Sunday in
# 2010 but that does not appear to have been the case previously.
# We'll assume that this will be the behavior from now on.
AnzacDayNonMondayized = anzac_day(end_date='2010')
AnzacDay = anzac_day(observance=sunday_to_monday, start_date='2010')

# When Easter Monday and Anzac Day coincided in 2011, Easter Tuesday was
# also observed as a public holiday. Note that this isn't defined as a
# rule, because it will happen next in 2095 (and then in  2163), and
# there isn't a great way to tell how this will be handled at that point.
EasterTuesday2011AdHoc = Timestamp('2011-04-26', tz='UTC')

QueensBirthday = Holiday(
    "Queen's Birthday",
    month=6,
    day=1,
    offset=[DateOffset(weekday=MO(2))],
)

LastTradingDayBeforeChristmas = Holiday(
    'Last Trading Day Before Christmas',
    month=12,
    day=24,
    start_date='2010',
    observance=previous_friday,
)
Christmas = christmas()
WeekendChristmas = weekend_christmas()
BoxingDay = boxing_day()
WeekendBoxingDay = weekend_boxing_day()

LastTradingDayOfCalendarYear = Holiday(
    'Last Trading Day Of Calendar Year',
    month=12,
    day=31,
    start_date='2010',
    observance=previous_friday,
)


class XASXExchangeCalendar(TradingCalendar):
    """
    Calendar for the Australian Securities Exchange in Sydney.

    Open Time: 10:00 AM, Australian Eastern Time
    Close Time: 4:00 PM, Australian Eastern Time

    Regularly-Observed Holidays:
      - New Year's Day
      - Australia Day
      - Good Friday
      - Easter Monday
      - Anzac Day
      - Queen's Birthday
      - Christmas Day
      - Boxing Day

    Early Closes:
      - Last trading day before Christmas
      - Last trading day of the calendar year
    """
    regular_early_close = time(14, 10)

    name = 'XASX'

    tz = timezone('Australia/Sydney')

    open_times = (
        (None, time(10, 1)),
    )

    close_times = (
        (None, time(16)),
    )

    @property
    def regular_holidays(self):
        return HolidayCalendar([
            NewYearsDay,
            AustraliaDay,
            GoodFriday,
            EasterMonday,
            AnzacDayNonMondayized,
            AnzacDay,
            QueensBirthday,
            Christmas,
            WeekendChristmas,
            BoxingDay,
            WeekendBoxingDay,
        ])

    @property
    def adhoc_holidays(self):
        return [EasterTuesday2011AdHoc]

    @property
    def special_closes(self):
        return [
            (
                self.regular_early_close,
                HolidayCalendar([
                    LastTradingDayBeforeChristmas,
                    LastTradingDayOfCalendarYear,
                ]),
            ),
        ]
